### $Id: Makefile.tpl 16 2007-10-04 15:27:41Z rea $

# Determining if we want to link with Sendmail's libsm.
WANTS_LIBSM=yes
.if ${OSNAME} == "FreeBSD"
# Get __FreeBSD_version
.if exists(/usr/include/sys/param.h)
FBSDVERSION!=     awk '/^\#define __FreeBSD_version/ {print $$3}' < /usr/include/sys/param.h
.elif exists(/usr/src/sys/sys/param.h)
FBSDVERSION!=     awk '/^\#define __FreeBSD_version/ {print $$3}' < /usr/src/sys/sys/param.h
.else
FBSDVERSION!=     sysctl -n kern.osreldate
.endif # exists(/usr/include/sys/param.h)
.if ${FBSDVERSION} >= 600000
WANTS_LIBSM=no
.endif # ${FBSDVERSION} >= 600000
.endif # ${OSNAME} = "FreeBSD"

RELEASETAG != cat $(.CURDIR)/RELEASE
RELEASEREV != cat $(.CURDIR)/REV 2>/dev/null || svn info | grep '^Revision:' | awk '{print $$2;}'
RELEASENAME = sm-filters-$(RELEASETAG)

CC = gcc
CFLAGS = -c -Iinclude -Wall -DRELEASE=\"$(RELEASENAME)\" -DREVISION=\"r$(RELEASEREV)\" -g
LFLAGS = -g
LIBS = -lmilter
.if ${WANTS_LIBSM} == yes
LIBS += -lsm
.endif
LIBSM = libsm.a
LIBMILTER = libmilter.a
MANPREF=.8

OBJS = ipc.o log.o utils.o drweb.o parse.o stringops.o hash.o crc32.o kasper.o rules.o
SRCS = ipc.c log.c utils.c drweb.c parse.c stringops.c hash.c crc32.c kasper.c rules.c
AIDS_OBJS = ipc.o log.o utils.o drweb.o parse.o stringops.o hash.o crc32.o kasper.o
AIDS_OBJS = ipc.o log.o utils.o drweb.o parse.o stringops.o hash.o crc32.o kasper.o
LOG_OBJS = ipc.o log.o stringops.o utils.o hash.o crc32.o
MAIL_OBJS = ipc.o log.o stringops.o utils.o hash.o crc32.o rules.o

all:	aids_filter log_filter mail_filter

install:	install-aids install-log install-mail

install-aids:	aids_filter
	mkdir -p $(BINDIR)
	cp aids_filter $(BINDIR)/aids_filter
	cp man/aids_filter.man $(MANDIR)/aids_filter$(MANPREF)
	gzip -f $(MANDIR)/aids_filter$(MANPREF)

install-log:	log_filter
	mkdir -p $(BINDIR)
	cp log_filter $(BINDIR)/log_filter
	cp man/log_filter.man $(MANDIR)/log_filter$(MANPREF)
	gzip -f $(MANDIR)/log_filter$(MANPREF)

install-mail:	mail_filter
	mkdir -p $(BINDIR)
	cp aids_filter $(BINDIR)/aids_filter

aids_filter:	$(AIDS_OBJS) aids_filter.o
		$(CC) $(LFLAGS) -pthread aids_filter.o $(AIDS_OBJS) -o aids_filter $(LIBS)

log_filter:	$(LOG_OBJS) log_filter.o
		$(CC) $(LFLAGS) -pthread log_filter.o $(LOG_OBJS) -o log_filter $(LIBS)

mail_filter:	$(MAIL_OBJS) mail_filter.o
		$(CC) $(LFLAGS) -pthread mail_filter.o $(MAIL_OBJS) -o mail_filter $(LIBS)

aids_filter.o:	aids_filter.c
		$(CC) $(CFLAGS) -I$(SENDMAIL)/include aids_filter.c -o aids_filter.o

log_filter.o:	log_filter.c
		$(CC) $(CFLAGS) -I$(SENDMAIL)/include log_filter.c -o log_filter.o

mail_filter.o:	mail_filter.c
		$(CC) $(CFLAGS) -I$(SENDMAIL)/include mail_filter.c -o mail_filter.o

.c.o:
		$(CC) $(CFLAGS) $< -o $@

clean:
		- rm -f $(OBJS) aids_filter.o aids_filter log_filter.o log_filter mail_filter mail_filter.o *.core libsm.a libmilter.a REV

distclean:	clean
		- rm -f Makefile Makefile.cfg Makefile.dep makeall .depend

tarball:
	rm -rf ../"$(RELEASENAME)"
	svn export . ../"$(RELEASENAME)"
	svn info | grep '^Revision:' | awk '{print $$2;}' > ../"$(RELEASENAME)"/REV
	(cd .. && tar -cvjf "$(RELEASENAME)".tar.bz2 "$(RELEASENAME)")
	rm -rf ../"$(RELEASENAME)"
