/*
 * $Id: aids_filter.c 14 2007-10-04 15:11:17Z rea $
 *
 * Copyright 2003-2007 Eygene A. Ryabinkin. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY EYGENE A. RYABINKIN "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EYGENE A. RYABINKIN OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <errno.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <sys/param.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <syslog.h>
#include <signal.h>
#include <string.h>
#include <strings.h>
#include <ctype.h>
#include "libmilter/mfapi.h"
#include "my.h"
#include "drweb.h"
#include "kasper.h"
#include "parse.h"
#include "hash.h"
#include "parse.h"

#if !defined(lint)
static const char vcsid[] = "$Id: aids_filter.c 14 2007-10-04 15:11:17Z rea $";
#endif

#define DEF_MY_SOCKET "/var/run/.aids_filter"
#define DEF_MY_TEMPLATE "/tmp/.virchkXXXXXX"
#define DEF_PROTO "local:"
#define MY_IDENT "aids_filter"
#define MAILPROG "/usr/sbin/sendmail"
#define DEFQDIRMODE 0700
#define INCLUDE_HEADER     "\n-- Infected message headers follows --\n"
#define INCLUDE_HEADER_END "-- End of headers --"

#define NUM_TOKENS (sizeof(tokens)/sizeof(TOKENS))
#define TOK_HOSTNAME 0
#define TOK_REASON   1
#define TOK_SENDER   2
#define TOK_RCPT     3

TOKENS tokens[]={
	{'h',NULL},
	{'r',NULL},
	{'S',NULL},
	{'R',"user(s)"}
};

typedef struct NAMELIST {
	char* name;
	void* next;
}NAMELIST,*NAMELIST_PTR;

typedef struct PRIV_DATA {
	int des;
	int checkit;
	int hdrlen;
	char name[MAXPATHLEN];
	char* from;
	char* helohost;
	NAMELIST root;
	NAMELIST_PTR last;
	TOKENS tokens[NUM_TOKENS];
}PRIV_DATA,*PRIV_DATA_PTR;

int Daemonize=0;
char temp_socket[MAXPATHLEN+sizeof(DEF_PROTO)];
int PrintPid,LLevel=0;
char Hostname[MAXHOSTNAMELEN];
char StripSymb[]="<>";

#define SUBJ_SIZE 1024

int UseWEB=0,UseKAV=0,Heuristic=0,WarnSender=0,WarnRecipients=0,CheckUsers=0;
int AllowWEB_RE=0,AllowWEB_WE=0,AllowWEB_MEM=0,AllowWEB_CRC=0,AllowWEB_Sock=0;
int AllowWEB_BIG=0,AllowWEB_COMP=0,AllowWEB_BC=0,IncludeHeaders=0;
char* UserTableFile=NULL;
HASH UsersHash;
long UTSize;
char* MySocket=DEF_MY_SOCKET;
char* DaemonSocket=NULL;
char* DaemonHost=NULL;
char* FileTemplate=DEF_MY_TEMPLATE;
char* SenderMessage=NULL;
char* RecipientsMessage=NULL;
char* Admin=NULL;
char* AdminMessage=NULL;
char* Quarantine=NULL;
long DaemonPort=0;

CONFIG_TOKENS conf_tokens[]={
	{"heuristic",9,CONFIG_TOKEN_FLAG,&Heuristic,0},
	{"usedrweb",8,CONFIG_TOKEN_FLAG,&UseWEB,0},
	{"usekav",6,CONFIG_TOKEN_FLAG,&UseKAV,0},
	{"warnsender",10,CONFIG_TOKEN_FLAG,&WarnSender,0},
	{"warnrecipients",14,CONFIG_TOKEN_FLAG,&WarnRecipients,0},
	{"sendermessage",13,CONFIG_TOKEN_STRING,&SenderMessage,0},
	{"recipientmessage",16,CONFIG_TOKEN_STRING,&RecipientsMessage,0},
	{"warnadmin",9,CONFIG_TOKEN_STRING,&Admin,0},
	{"adminmessage",12,CONFIG_TOKEN_STRING,&AdminMessage,0},
	{"socket",6,CONFIG_TOKEN_STRING,&MySocket,0},
	{"daemonsocket",12,CONFIG_TOKEN_STRING,&DaemonSocket,0},
	{"daemonhost",10,CONFIG_TOKEN_STRING,&DaemonHost,0},
	{"filetemplate",12,CONFIG_TOKEN_STRING,&FileTemplate,0},
	{"daemonport",10,CONFIG_TOKEN_LONG,&DaemonPort,0},
	{"userdb",6,CONFIG_TOKEN_STRING,&UserTableFile,0},
	{"quarantine",10,CONFIG_TOKEN_STRING,&Quarantine,0},
	{"allowreaderror",14,CONFIG_TOKEN_FLAG,&AllowWEB_RE,0},
	{"allowwriterror",14,CONFIG_TOKEN_FLAG,&AllowWEB_WE,0},
	{"allowmemerror",13,CONFIG_TOKEN_FLAG,&AllowWEB_MEM,0},
	{"allowcrcerror",13,CONFIG_TOKEN_FLAG,&AllowWEB_CRC,0},
	{"allowsocketerror",16,CONFIG_TOKEN_FLAG,&AllowWEB_Sock,0},
	{"allowtoobigerror",16,CONFIG_TOKEN_FLAG,&AllowWEB_BIG,0},
	{"allowcomperror",14,CONFIG_TOKEN_FLAG,&AllowWEB_COMP,0},
	{"allowbadcall",12,CONFIG_TOKEN_FLAG,&AllowWEB_BC,0},
	{"includeheaders",14,CONFIG_TOKEN_FLAG,&IncludeHeaders,0}
};

void logVirus(PRIV_DATA_PTR,TOKENS_PTR,long);

sfsistat mlfi_connect(SMFICTX* ctx,char* hostname,_SOCK_ADDR* hostaddr)
/******************************************************************************
 * Called for each SMTP connection.                                           *
 * Here we do memory allocation for this connection and some initialisation.  *
 ******************************************************************************/
{
	PRIV_DATA_PTR privData;
	long i;

	LogDebug(8,"*** New connection initiated. ***");
	if( !(privData=(void*)malloc(sizeof(PRIV_DATA))) )
	{
		LogError(-1,"%s,%d: unable to allocate memory",__FILE__,__LINE__);
		smfi_setpriv(ctx,NULL);
		return SMFIS_TEMPFAIL;
	}
	privData->helohost=NULL;
	privData->from=NULL;
	/* If we're not using 'User Table', e.g. checking mail only for/from
	 * specified users, default behaviour is to check this mail unconditionally.
	 * But if table exists, the message will be checked only if recipient's
	 * or sender's address will be in this table.
	 */
	if( CheckUsers )
		privData->checkit=0;
	else
		privData->checkit=1;
	for(i=0;i<NUM_TOKENS;i++)
	{
		privData->tokens[i].name=tokens[i].name;
		privData->tokens[i].value=tokens[i].value;
	}
	/* Set up recipients list. */
	privData->root.next=NULL;
	privData->last=&(privData->root);
	smfi_setpriv(ctx,privData);
	/* Obtaining temp file for body of current message. */
	strlcpy(privData->name, FileTemplate, sizeof(privData->name));
	privData->des=mkstemp(privData->name);
	privData->hdrlen=0;
	return SMFIS_CONTINUE;
}

sfsistat mlfi_helo(SMFICTX* ctx,char* helohost)
/******************************************************************************
 * Called for HELO/EHLO command issued.                                       *
 ******************************************************************************/
{
	PRIV_DATA_PTR privData;
	long len;

	privData=smfi_getpriv(ctx);
	if( privData == NULL )
		return SMFIS_TEMPFAIL;
	len=strlen(helohost)+1;
	if( !(privData->helohost=(char*)malloc(len*sizeof(char))) )
	{
		LogError(-1,"Cannot allocate memory for helohost data.");
		return SMFIS_TEMPFAIL;
	}
	strlcpy(privData->helohost, helohost, len);

	LogDebug(8,"HELO `%s'.",privData->helohost);

	return SMFIS_CONTINUE;
}

sfsistat mlfi_envfrom(SMFICTX* ctx,char** argv)
/******************************************************************************
 * FROM handler.                                                              *
 ******************************************************************************/
{
	PRIV_DATA_PTR privData;
	long len;

	privData=smfi_getpriv(ctx);
	if( privData == NULL )
		return SMFIS_TEMPFAIL;
	len=strlen(argv[0])+1;
	if( !(privData->from=(char*)malloc(len*sizeof(char))) )
	{
		LogError(-1,"Cannot allocate memory for from field");
		return SMFIS_TEMPFAIL;
	}
	strlcpy(privData->from, argv[0], len);
	StripSymbols(privData->from,StripSymb,sizeof(StripSymb)/sizeof(char)-1);
	len=0;
	while( privData->from[len] )
	{
		privData->from[len]=tolower(privData->from[len]);
		len++;
	}

	LogDebug(8,"MAIL FROM '%s'.",privData->from);

	if( CheckUsers && privData->checkit == 0 )
		if( InHash(&UsersHash,privData->from) != -1 )
			privData->checkit=1;

	return SMFIS_CONTINUE;
}

sfsistat mlfi_envrcpt(SMFICTX* ctx,char** argv)
/******************************************************************************
 * RCPT handler.                                                              *
 ******************************************************************************/
{
	PRIV_DATA_PTR privData;
	long len;

	privData=smfi_getpriv(ctx);
	if( privData == NULL )
		return SMFIS_TEMPFAIL;
	if( !(privData->last->next=(NAMELIST_PTR)malloc(sizeof(NAMELIST))) )
	{
		LogError(-1,"Cannot allocate memory for next recipient.");
		return SMFIS_TEMPFAIL;
	}
	privData->last=privData->last->next;
	privData->last->next=NULL;
	privData->last->name=NULL;
	len=strlen(argv[0])+1;
	if( !(privData->last->name=(char*)malloc(len*sizeof(char))) )
	{
		LogError(-1,"Cannot allocate memory for recipient name.");
		return SMFIS_TEMPFAIL;
	}
	strlcpy(privData->last->name, argv[0], len);
	StripSymbols(privData->last->name,StripSymb,
	sizeof(StripSymb)/sizeof(char)-1);
	len=0;
	while( privData->last->name[len] )
	{
		privData->last->name[len]=tolower(privData->last->name[len]);
		len++;
	}

	LogDebug(8,"RCPT TO '%s'.",privData->last->name);

	if( CheckUsers && privData->checkit == 0 )
		if( InHash(&UsersHash,privData->last->name) != -1 )
		{
			LogDebug(8,"Checking message.");
			privData->checkit=1;
		}

	return SMFIS_CONTINUE;
}

sfsistat mlfi_header(SMFICTX* ctx,char* name,char* value)
/******************************************************************************
 * Called for each header line.                                               *
 ******************************************************************************/
{
	PRIV_DATA_PTR privData;

	privData=smfi_getpriv(ctx);
	if( privData == NULL )
		return SMFIS_TEMPFAIL;

	privData->hdrlen += strlen(name) + strlen(value) + 3;
	write(privData->des, name, strlen(name));
	write(privData->des, ": ", 2);
	write(privData->des, value, strlen(value));
	write(privData->des, "\n", 1);
	return SMFIS_CONTINUE;
}

sfsistat mlfi_eoh(SMFICTX* ctx)
/******************************************************************************
 * Called when all headers were processed.                                    *
 ******************************************************************************/
{
	PRIV_DATA_PTR privData;

	privData=smfi_getpriv(ctx);
	if( privData == NULL )
		return SMFIS_TEMPFAIL;
	write(privData->des,"\n",1);

	return SMFIS_CONTINUE;
}

sfsistat mlfi_body(SMFICTX* ctx,unsigned char* bodychunk,size_t len)
/******************************************************************************
 * Called for each body chunk.                                                *
 ******************************************************************************/
{
	PRIV_DATA_PTR privData;

	privData=smfi_getpriv(ctx);
	if( !privData )
	{
		LogError(-1,"%s,%d: NULL private data in mlfi_body()",__FILE__,__LINE__);
		return SMFIS_TEMPFAIL;
	}

	if( privData->checkit )
		/* Adding this chunk to file. */
		write(privData->des,bodychunk,len);

	return SMFIS_CONTINUE;
}

sfsistat mlfi_eom(SMFICTX* ctx)
/******************************************************************************
 * Called when message is completed. Last chance to make any modifications.   *
 ******************************************************************************/
{
	PRIV_DATA_PTR privData;
	NAMELIST_PTR ptr;
	int s,res,vnumber,flags,free_reason,free_rcpts,i;
	char** vnames;
	char* reason,*rcpts;

	LogDebug(8,"%s,%d: mlfi_eom() called",__FILE__,__LINE__);
	privData=smfi_getpriv(ctx);
	if( !privData )
	{
		LogError(-1,"%s,%d: NULL private data in mlfi_eom()",__FILE__,__LINE__);
		return SMFIS_TEMPFAIL;
	}
	/* Close message file first. A small workaround
	 * for 'too many open files' problem. */
	close(privData->des);
	if( privData->checkit == 0 )
	/* Said not to check this letter. */
		return SMFIS_CONTINUE;

	/* Going to check saved body. */
	if( UseWEB )
	{
		LogDebug(8,"Checking file `%s' with DrWeb.",privData->name);
		s=openUnixSocket(DaemonSocket);
		flags=DRWEBD_RETURN_VIRUSES;
		if( Heuristic )
			flags|=DRWEBD_HEURISTIC_ON;
		res=checkDiskFileDrWeb(s,privData->name,flags,&vnames,&vnumber);
		close(s);
		if( res & DERR_MASK )
		{
			free_reason=0;
			reason=DrWebError(res,vnames,vnumber);
			if( (res & DERR_MASK & DERR_KNOWN_VIRUS) )
			{
				if( reason )
					free_reason=1;
				else
					reason="infected by virus";
			}
			if( !reason )
				return SMFIS_CONTINUE;
			/* Setup dynamic values for my_printf(). */
			i=0;
			ptr=(NAMELIST_PTR)(privData->root.next);
			while(ptr)
			{
				i += strlen(ptr->name) + 1;
				ptr=ptr->next;
			}
			if( i == 1 || !(rcpts=(char*)malloc(i*sizeof(char))) )
			{
				free_rcpts=0;
				rcpts="some recipients";
			}
			else
			{
				free_rcpts=1;
				rcpts[0]='\0';
				ptr=(NAMELIST_PTR)(privData->root.next);
				while(ptr)
				{
					strlcat(rcpts, ptr->name, i);
					strlcat(rcpts, ",", i);
					ptr=ptr->next;
				}
				rcpts[i-1]='\0';
			}
			privData->tokens[TOK_SENDER].value=privData->from;
			privData->tokens[TOK_REASON].value=reason;
			privData->tokens[TOK_RCPT].value=rcpts;
			logVirus(privData,privData->tokens,NUM_TOKENS);
			for(s=0;s<vnumber;s++)
				free(vnames[s]);
			free(vnames);
			if( free_reason )
				free(reason);
			return SMFIS_DISCARD;
		}
	}
	else if( UseKAV )
	{
		LogDebug(8,"Checking file `%s' with KAV.",privData->name);
		s=openUnixSocket(DaemonSocket);
		res=checkFileKAV(s,privData->name);
		close(s);
		if( res != 0 )
		{
			free_reason=0;
			reason=KAVError(res);
			if( !reason )
				return SMFIS_CONTINUE;
			/* Setup dynamic values for my_printf(). */
			i=0;
			ptr=(NAMELIST_PTR)(privData->root.next);
			while(ptr)
			{
				i += strlen(ptr->name) + 1;
				ptr=ptr->next;
			}
			if( i == 1 || !(rcpts=(char*)malloc(i*sizeof(char))) )
			{
				free_rcpts=0;
				rcpts="some recipients";
			}
			else
			{
				free_rcpts=1;
				rcpts[0]='\0';
				ptr=(NAMELIST_PTR)(privData->root.next);
				while(ptr)
				{
					strlcat(rcpts, ptr->name, i);
					strlcat(rcpts, ",", i);
					ptr=ptr->next;
				}
				rcpts[i-1]='\0';
			}
			privData->tokens[TOK_SENDER].value=privData->from;
			privData->tokens[TOK_REASON].value=reason;
			privData->tokens[TOK_RCPT].value=rcpts;
			logVirus(privData,privData->tokens,NUM_TOKENS);
			if( free_reason )
				free(reason);
			if( free_rcpts )
				free(rcpts);
			return SMFIS_DISCARD;
		}
	}

	return SMFIS_CONTINUE;
}

sfsistat mlfi_abort(SMFICTX* ctx)
/******************************************************************************
 * Called when message being aborted.                                         *
 ******************************************************************************/
{
	return SMFIS_CONTINUE;
}

sfsistat mlfi_close(SMFICTX* ctx)
/******************************************************************************
 * The very last handler.                                                     *
 ******************************************************************************/
{
	PRIV_DATA_PTR privData;

	NAMELIST_PTR ptr,temp;
	privData=smfi_getpriv(ctx);
	if( !privData )
		return SMFIS_TEMPFAIL;
	unlink(privData->name);
	ptr=(NAMELIST_PTR)(privData->root.next);
	while(ptr)
	{
		if( ptr->name )
			free(ptr->name);
		temp=ptr;
		ptr=temp->next;
		free(temp);
	}

	if( privData->from )
		free(privData->from);
	if( privData->helohost )
		free(privData->helohost);
	if( privData )
		free(privData);
	smfi_setpriv(ctx,NULL);
	LogDebug(8,"*** Data freed, connection closed. ***");

	return SMFIS_CONTINUE;
}

#define WRBUFSIZE  16*1024

void quarantineFile(char* name)
/******************************************************************************
 * Moves a named file to a quarantine.                                        *
 ******************************************************************************/
{
	int des,infdes;
	long mesnum,next,size;
	time_t tim;
	struct stat sb;
	struct tm ltime;
	int tm_mday,tm_mon,tm_year;
	char ctlfile[MAXPATHLEN],buf[WRBUFSIZE];

	if (strlen(Quarantine) + 20 > MAXPATHLEN)
	{
		LogError(-1,"Cannot put infected message to quarantine: path too long.");
		return;
	}
	tim=time(NULL);
	localtime_r(&tim,&ltime);
	strlcpy(ctlfile, Quarantine, sizeof(ctlfile));
	strlcat(ctlfile, "/.ctl", sizeof(ctlfile));
	if( stat(ctlfile,&sb) || !(sb.st_mode & S_IFREG) )
	{
		LogDebug(8,"No control file `%s'. Creating.",ctlfile);
		tm_mday=ltime.tm_mday;
		tm_mon=ltime.tm_mon;
		tm_year=ltime.tm_year;
		mesnum=0;
		next=1;
		unlink(ctlfile);
		if( (des=open(ctlfile,O_WRONLY|O_CREAT|O_EXLOCK,0600)) < 0 )
		{
			LogError(-1,"Cannot create control file `%s': %s.",
			ctlfile,strerror(errno));
			return;
		}
		write(des,&next,sizeof(next));
		write(des,&tm_mday,sizeof(tm_mday));
		write(des,&tm_mon,sizeof(tm_mon));
		write(des,&tm_year,sizeof(tm_year));
		flock(des,LOCK_UN);
		close(des);
		LogDebug(8,"Control file created.");
	}
	else
	{
		if( (des=open(ctlfile,O_RDWR|O_EXLOCK)) < 0 )
		{
			LogError(-1,"Cannot open control file `%s': %s.",
			ctlfile,strerror(errno));
			return;
		}
		read(des,&mesnum,sizeof(mesnum));
		read(des,&tm_mday,sizeof(tm_mday));
		read(des,&tm_mon,sizeof(tm_mon));
		read(des,&tm_year,sizeof(tm_year));
		if( tm_mday == ltime.tm_mday && tm_mon == ltime.tm_mon &&
				tm_year == ltime.tm_year )
		{
			next=mesnum+1;
			LogDebug(8,"Next message number is %ld.",next);
		}
		else
		{
			mesnum=0;
			next=1;
			tm_mday=ltime.tm_mday;
			tm_mon=ltime.tm_mon;
			tm_year=ltime.tm_year;
			LogDebug(8,"Day changed. Next message number is %ld.",next);
		}
		/* Rewriting control file. */
		lseek(des,0,SEEK_SET);
		write(des,&next,sizeof(next));
		write(des,&tm_mday,sizeof(tm_mday));
		write(des,&tm_mon,sizeof(tm_mon));
		write(des,&tm_year,sizeof(tm_year));
		flock(des,LOCK_UN);
		close(des);
		LogDebug(8,"Control file updated.");
	}

	snprintf(ctlfile,sizeof(ctlfile),"%s/%02d.%02d.%04d_%ld",Quarantine,
	tm_mday,tm_mon+1,tm_year+1900,mesnum);
	LogDebug(8,"Moving `%s' to `%s'.",name,ctlfile);
	if( (des=open(ctlfile,O_CREAT|O_WRONLY,0600)) < 0 )
	{
		LogError(-1,"Cannot create target quarantine file `%s'.",ctlfile);
		return;
	}
	if( (infdes=open(name,O_RDONLY)) < 0 )
	{
		close(des);
		LogError(-1,"Cannot open source file with infected message `%s'.",name);
		return;
	}

	while( (size=read(infdes,buf,sizeof(buf))) > 0 )
	{
		LogDebug(8,"Writing %ld bytes.",size);
		write(des,buf,size);
	}

	close(des);
	close(infdes);
	
	return;
}

void includeHeaders(int des,char* fname,int len)
/*
 * Includes original message headers.
 */
{
	char buffer[1024];
	int nbytes,maildes;

	if( (maildes=open(fname,O_RDONLY)) < 0 )
		return;
	lseek(des,0,SEEK_SET);
	write(des,INCLUDE_HEADER,sizeof(INCLUDE_HEADER)-1);
	while( (nbytes=read(maildes,buffer,sizeof(buffer))) > 0 )
	{
		if( nbytes < len )
		{
			write(des,buffer,nbytes);
			write(1,buffer,nbytes);
		}
		else
		{
			write(des,buffer,len);
			write(1,buffer,len);
			break;
		}
		len-=nbytes;
	}
	write(des,INCLUDE_HEADER_END,sizeof(INCLUDE_HEADER_END)-1);
	close(maildes);

	return;
}

void logVirus(PRIV_DATA_PTR pData,TOKENS_PTR tokens,long toknum)
/******************************************************************************
 * Logs infected mail.                                                        *
 ******************************************************************************/
{
	NAMELIST_PTR ptr;
	int des;
	long pid;
	
	LogError(-1,"Message from `%s' to `%s' rejected, reason is '%s'.",
	tokens[TOK_SENDER].value,tokens[TOK_RCPT].value,
	tokens[TOK_REASON].value);

	if( Quarantine )
		quarantineFile(pData->name);

	if( WarnRecipients )
	{
		ptr=pData->root.next;
		while( ptr )
		{
			LogDebug(8,"Sending a warning to recipient '%s'.",ptr->name);
			des=SpawnMail(MAILPROG,&pid,ptr->name);
			my_printf(des,RecipientsMessage,tokens,toknum);
			if( IncludeHeaders )
				includeHeaders(des,pData->name,pData->hdrlen);
			/* Terminator. */
			write(des,"\n.\n",3);
			close(des);
			waitpid(pid,&des,0);
			ptr=ptr->next;
		}
	}

	if( WarnSender )
	{
		LogDebug(8,"Sending a warning to sender '%s'.",pData->from);
		des=SpawnMail(MAILPROG,&pid,pData->from);
		my_printf(des,SenderMessage,tokens,toknum);
		if( IncludeHeaders )
			includeHeaders(des,pData->name,pData->hdrlen);
		/* Terminator. */
		write(des,"\n.\n",3);
		close(des);
		waitpid(pid,&des,0);
	}

	if( Admin )
	{
		LogDebug(8,"Sending a warning to admin '%s'.",Admin);
		des=SpawnMail(MAILPROG,&pid,Admin);
		my_printf(des,AdminMessage,tokens,toknum);
		if( IncludeHeaders )
			includeHeaders(des,pData->name,pData->hdrlen);
		/* Terminator. */
		write(des,"\n.\n",3);
		close(des);
		waitpid(pid,&des,0);
	}

	return;
}

struct smfiDesc smfilter = {
	"StopAIDS",          /* Filter name. */
	SMFI_VERSION,        /* Version. */
	0,                   /* Flags. */
	mlfi_connect,        /* Connection handler. */
	mlfi_helo,           /* HELO command handler. */
	mlfi_envfrom,        /* Sender envelope handler. */
	mlfi_envrcpt,        /* Recipient envelope handler. */
	mlfi_header,         /* Header handler. */
	mlfi_eoh,            /* End-of-header handler. */
	mlfi_body,           /* Body handler. */
	mlfi_eom,            /* End-of-message handler. */
	mlfi_abort,          /* Message abort handler. */
	mlfi_close           /* Cleanup handler. */
};

int sig_handler(int s)
/******************************************************************************
 * Default handler.                                                           *
 ******************************************************************************/
{
	unlink(MySocket);
	return 0;
}

void usage(char *progname)
{
	printf("This is Dr. Web filter for sendmail, version '" RELEASE "'.\n"
"Usage: %s [options]\n"
"\n"
"Options (classic getopt style):\n"
"-F file		read configuration from the given file\n"
"-d		become daemon\n"
"-h		this help message\n"
"-p		print PID to stdout\n"
"-v level	set specified verbosity level\n"
	    ,progname);
	exit(0);
}

void parseCmdLine(int,char*[]);
int main(int argc,char* argv[])
{
	int pid;

	parseCmdLine(argc,argv);
	gethostname(Hostname, sizeof(Hostname));
	if (!Hostname[0])
		strlcpy(Hostname, "Amnesiac", sizeof(Hostname));
	tokens[TOK_HOSTNAME].value=Hostname;

	if( Daemonize )
	{
		while( (pid=fork()) == -1 )
			sleep(5);
		if( pid )
			exit(0);
		if( PrintPid )
			printf("%ld\n",(long)getpid());
		close(0); close(1); close(2);
		LogOpenSyslog(MY_IDENT,LOG_MAIL);
	}
	else
	{
		LogOpenStderr();
		if( PrintPid )
			printf("%ld\n",(long)getpid());
	}
	SetLogLevel(LLevel);

	if( !isSocket(DaemonSocket) )
	{
		LogError(-1,"`%s' is not a valid socket",DaemonSocket);
		return -1;
	}

	unlink(MySocket);
	strlcpy(temp_socket, DEF_PROTO, sizeof(temp_socket));
	strlcat(temp_socket, MySocket, sizeof(temp_socket));
	if( smfi_setconn(temp_socket) == MI_FAILURE )
	{
		LogError(-1,"smfi_setconn(\'%s\') failed",MySocket);
		return -1;
	}
	if( smfi_register(smfilter) == MI_FAILURE )
	{
		LogError(-1,"Cannot register filter");
		return -1;
	}

	LogDebug(8,"Starting main loop.");
	return smfi_main();
}

void parseCmdLine(int argc,char* argv[])
{
	long size,i;
	char result;
	FILE* fp;
	struct stat sb;

	Daemonize=0;
	PrintPid=0;

	while( (result=getopt(argc,argv,"dhpF:v:")) != -1 )
	{
		if( result == 'h' )
			usage(argv[0]);
		else if( result == 'd' )
			Daemonize=1;
		else if( result == 'p' )
			PrintPid=1;
		else if( result == 'v' )
			LLevel=atoi(optarg);
		else if( result == 'F' )
		{
			if( !(fp=fopen(optarg,"r")) )
			{
				printf("Unable to open configuration file '%s'.\n",optarg);
				exit(-1);
			}
			parse_config(fp,conf_tokens,sizeof(conf_tokens)/sizeof(CONFIG_TOKENS));
			fclose(fp);
			/* Check parameters. */
			if( UseWEB && UseKAV )
			{
				printf("Only one antivirus engine must be specified.\n");
				exit(-2);
			}

			if( WarnSender )
			{
				if( SenderMessage == NULL )
				{
					printf("Asked to warn sender, but no message template specified.\n");
					exit(-2);
				}
				else
				{
					if( !(fp=fopen(SenderMessage,"r")) )
					{
						printf("Cannot open sender message template file '%s'.\n",
						SenderMessage);
						exit(-1);
					}
					free(SenderMessage);
					fseek(fp,0,SEEK_END);
					size=ftell(fp);
					fseek(fp,0,SEEK_SET);
					if( !(SenderMessage=(char*)malloc((size+1)*sizeof(char))) )
					{
						printf("Unable to allocate memory for sender message template.\n");
						exit(-3);
					}
					bzero(SenderMessage,size+1);
					for(i=0;i<size;i++)
						SenderMessage[i]=fgetc(fp);
					fclose(fp);
				}
			}

			if( WarnRecipients )
			{
				if( RecipientsMessage == NULL )
				{
					printf("Asked to warn recipients, but no message template specified."
								 "\n");
					exit(-2);
				}
				else
				{
					if( !(fp=fopen(RecipientsMessage,"r")) )
					{
						printf("Cannot open recipients message template file '%s'.\n",
						RecipientsMessage);
						exit(-1);
					}
					free(RecipientsMessage);
					fseek(fp,0,SEEK_END);
					size=ftell(fp);
					fseek(fp,0,SEEK_SET);
					if( !(RecipientsMessage=(char*)malloc((size+1)*sizeof(char))) )
					{
						printf("Unable to allocate memory for recipients message template."
									 "\n");
						exit(-3);
					}
					bzero(RecipientsMessage,size+1);
					for(i=0;i<size;i++)
						RecipientsMessage[i]=fgetc(fp);
					fclose(fp);
				}
			}

			if( Admin )
			{
				if( AdminMessage == NULL )
				{
					printf("Asked to warn adminnistrator, but no message template specified."
								 "\n");
					exit(-2);
				}
				else
				{
					if( !(fp=fopen(AdminMessage,"r")) )
					{
						printf("Cannot open admin message template file '%s'.\n",
						AdminMessage);
						exit(-1);
					}
					free(AdminMessage);
					fseek(fp,0,SEEK_END);
					size=ftell(fp);
					fseek(fp,0,SEEK_SET);
					if( !(AdminMessage=(char*)malloc((size+1)*sizeof(char))) )
					{
						printf("Unable to allocate memory for admin message template."
									 "\n");
						exit(-3);
					}
					bzero(AdminMessage,size+1);
					for(i=0;i<size;i++)
						AdminMessage[i]=fgetc(fp);
					fclose(fp);
				}
			}

			if( UserTableFile )
			{
				CheckUsers=1;
				if( !(fp=fopen(UserTableFile,"r")) )
				{
					printf("Unable to open user table file '%s'.\n",UserTableFile);
					exit(-3);
				}
				if( ReadUsers(fp,&UsersHash) == -1 )
				{
					printf("Unable to allocate memory for users table.\n");
					exit(-3);
				}
				fclose(fp);
			}

			if( Quarantine )
			{
				if( stat(Quarantine,&sb) )
				{
					printf("Can't stat `%s': %s.\n",Quarantine,strerror(errno));
					exit(-1);
				}
				if( !(sb.st_mode&S_IFDIR) )
				{
					printf("Quarantine `%s' is not a directory.\n",Quarantine);
					exit(-1);
				}
				if( (sb.st_mode&0777) != DEFQDIRMODE )
				{
					printf("Quarantine directory `%s' must be mode %o.\n",Quarantine,
					DEFQDIRMODE);
					exit(-1);
				}
			}

		}
	}
	return;
}
