/*
 * $Id: drweb.c 13 2007-10-04 14:54:56Z rea $
 *
 * Copyright 2003-2007 Eygene A. Ryabinkin. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY EYGENE A. RYABINKIN "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EYGENE A. RYABINKIN OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#if !defined(lint)
static const char vcsid[] = "$Id: drweb.c 13 2007-10-04 14:54:56Z rea $";
#endif

#include <unistd.h>
#include <string.h>
#include <strings.h>
#include <stdlib.h>
#include "my.h"
#include "drweb.h"

int checkFileDrWeb(int Sock,char* Name,char* Contents,int Size,int Options,
                   char*** vnames,int* vnumber)
/******************************************************************************
 * Feeds the file to an instance of 'drwebd' through socket.                  *
 * Call:                                                                      *
 *  Sock - socket to communicate with 'drwebd'.                               *
 *  Name - description for file, not neccesarily it's real name.              *
 *  Contents - buffer with our file.                                          *
 *  Size - size of `Contents' buffer.                                         *
 *  Options - options for 'drwebd'.                                           *
 ******************************************************************************/
{
	int word;

	/* 'Scan file' command. */
	word=DRWEBD_SCAN_CMD;
	writeInt(Sock,word);
	/* Scan options. */
	writeInt(Sock,Options);
	/* Length of file description. */
	word=strlen(Name);
	writeInt(Sock,word);
	/* The description itself. */
	write(Sock,Name,word);
	/* Size of file. */
	writeInt(Sock,Size);
	/* File. */
	write(Sock,Contents,Size);
	/* Read return code. */
	readInt(Sock,&word);
	LogDebug(9,"DRWEBD return code is %d.",word);
	/* Analyse output. */
	analyseDrWebOutput(Sock,Options,vnames,vnumber);

	return word;
}

int checkDiskFileDrWeb(int Sock,char* Name,int Options,char*** vnames,
                       int* vnumber)
/******************************************************************************
 * Feeds the filename to an instance of 'drwebd' through socket.              *
 * Call:                                                                      *
 *  Sock - socket to communicate with 'drwebd'.                               *
 *  Name - description for file, not neccesarily it's real name.              *
 *  Options - options for 'drwebd'.                                           *
 ******************************************************************************/
{
	int word;

	/* 'Scan file' command. */
	word=DRWEBD_SCAN_CMD;
	writeInt(Sock,word);
	/* Scan options. */
	writeInt(Sock,Options);
	/* Length of file name. */
	word=strlen(Name);
	writeInt(Sock,word);
	/* The description itself. */
	write(Sock,Name,word);
	/* Size of file is 0. */
	word=0;
	writeInt(Sock,word);
	/* Read return code. */
	readInt(Sock,&word);
	LogDebug(9,"DRWEBD return code is %d.",word);
	/* Analyse output. */
	analyseDrWebOutput(Sock,Options,vnames,vnumber);

	return word;
}

void analyseDrWebOutput(int Sock,int Options,char*** vnames,int* vnumber)
/******************************************************************************
 * Analyses output from 'drwebd', that depends on given options.              *
 * Call:                                                                      *
 *  Sock - 'drwebd' socket.                                                   *
 *  Options - options previously passed to 'drwebd'.                          *
 ******************************************************************************/
{
	char virname[VIR_LEN],repline[REP_LEN];
	int i,n,len,what;

	if( (Options & DRWEBD_RETURN_VIRUSES) == DRWEBD_RETURN_VIRUSES )
	{
		/* Read number of viruses, that were found. */
		readInt(Sock,&n);
		*vnumber=n;
		if( !(*vnames=(char**)malloc(n*sizeof(char*))) )
		{
			LogError(-1,"Cannot allocate memory for virus names");
		}
		LogDebug(9,"Found %ld viruses.",n);
		/* Read each virus description. */
		for(i=0;i<n;i++)
		{
			readInt(Sock,&len);
			if( len > VIR_LEN )
				len=VIR_LEN;
			read(Sock,virname,len);
			if( *vnames )
			{
				if( !((*vnames)[i]=(char*)malloc(len*sizeof(char))) )
				{
					LogError(-1,"Cannot allocate memory for virus name number %ld",i);
				}
				else
					strlcpy((*vnames)[i], virname, len);
			}
			LogDebug(9,"Virus '%s' detected.",virname);
		}
	}
	if( (Options & DRWEBD_RETURN_REPORT) == DRWEBD_RETURN_REPORT )
	{
		/* Read number of lines in the report. */
		readInt(Sock,&n);
		LogDebug(9,"Got report, %ld lines.",n);
		/* Read each report line. */
		for(i=0;i<n;i++)
		{
			readInt(Sock,&len);
			if( len > REP_LEN )
				len=REP_LEN;
			read(Sock,repline,len);
			LogDebug(9,"%s",repline);
		}
	}
	if( (Options & DRWEBD_RETURN_CODES) == DRWEBD_RETURN_CODES )
	{
		/* Read number of returned codes. */
		readInt(Sock,&n);
		LogDebug(9,"%ld return codes.",n);
		/* Read return codes and it's values. */
		for(i=0;i<n;i++)
		{
			readInt(Sock,&what);
			read(Sock,&len,sizeof(what));
			LogDebug(9,"What = %d, value is %d.",what,len);
		}
	}

	return;
}

extern int AllowWEB_RE,AllowWEB_WE,AllowWEB_MEM,AllowWEB_CRC;
extern int AllowWEB_Sock,AllowWEB_BIG,AllowWEB_COMP,AllowWEB_BC;

char* DrWebError(long code,char** vnames,int vnumber)
/******************************************************************************
 ******************************************************************************/
{
	char* ptr;
	long size,i;

	code&=DERR_MASK;
	if( code & DERR_KNOWN_VIRUS )
	{
		size=0;
		for(i=0;i<vnumber;i++)
			size += 1 + strlen(vnames[i]);
		if( !(ptr=(char*)malloc(size*sizeof(char))) )
			return NULL;
		strlcpy(ptr, vnames[0], size);
		for(i=1;i<vnumber;i++)
		{
			strlcat(ptr, ",", size);
			strlcat(ptr, vnames[i], size);
		}
		return ptr;
	}
	if( code & DERR_UNKNOWN_VIRUS )
		return "infected by unknown virus";
	if( code & DERR_VIRUS_MODIFICATION )
		return "infected by virus modification";
	if( code & DERR_READ_ERR && AllowWEB_RE == 0 )
		return "read error, possibly bad message format";
	if( code & DERR_WRITE_ERR && AllowWEB_WE == 0 )
		return "DrWeb write error";
	if( code & DERR_NOMEMORY && AllowWEB_MEM == 0 )
		return "DrWeb was unable to allocate memory";
	if( code & DERR_CRC_ERROR && AllowWEB_CRC == 0 )
		return "checksum error occured while checking the message";
	if( code & DERR_READSOCKET && AllowWEB_Sock == 0 )
		return "DrWeb transport subsystem error";
	if( code & DERR_TOO_BIG && AllowWEB_BIG == 0 )
		return "our message was too big for DrWeb";
	if( code & DERR_TOO_COMPRESSED && AllowWEB_COMP == 0 )
		return "certain parts of message have too high compression";
	if( code & DERR_BAD_CALL && AllowWEB_BC == 0 )
		return "DrWeb issued a bad call";

	return NULL;
}
