/*
 * $Id: hash.c 13 2007-10-04 14:54:56Z rea $
 *
 * Copyright 2003-2007 Eygene A. Ryabinkin. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY EYGENE A. RYABINKIN "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EYGENE A. RYABINKIN OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#if !defined(lint)
static const char vcsid[] = "$Id: hash.c 13 2007-10-04 14:54:56Z rea $";
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include "hash.h"
#include "crc32.h"

/* TODO: add element deletion. */

unsigned long __HashFunc(char* key,long max)
{
	unsigned long retval;

	retval=crc32(0xFFFFFFFF, key, strlen(key))%max;
	#ifdef DEBUG
	printf("CRC32 for string '%s' and nmax = %ld is %ld.\n",key,max,retval);
	#endif

	return retval;
}

int CreateHash(HASH_PTR hash,long size)
{
	long i;

	if( !(hash->elements=(void**)malloc(size*sizeof(void*))) )
	{
		return -1;
	}
	if( !(hash->hentry=(HASH_ENTRY_PTR*)malloc(size*sizeof(HASH_ENTRY_PTR))) )
	{
		free(hash->elements);
		return -1;
	}
	hash->size=size;
	hash->free=0;

	for(i=0;i<size;i++)
		hash->hentry[i]=NULL;

	return 0;
}

void DeleteHash(HASH_PTR hash)
{
	long i;
	HASH_ENTRY_PTR entry,next;
	free(hash->elements);
	
	for(i=0;i<hash->size;i++)
	{
		entry=hash->hentry[i];
		while( entry )
		{
			next=entry->next;
			free(entry->key);
			free(entry);
			entry=next;
		}
	}
	free(hash->hentry);

	return;
}

long InHash(HASH_PTR hash,char* key)
{
	long hval;
	HASH_ENTRY_PTR entry;

	hval=__HashFunc(key,hash->size);
	entry=hash->hentry[hval];
	#ifdef DEBUG
	printf("InHash(): checking entry number %ld for key '%s'.\n",hval,key);
	#endif

	while( entry )
	{
		#ifdef DEBUG
		printf("\tKey is '%s', idx = %ld.\n",entry->key,entry->n);
		#endif
		if (!strcmp(key, entry->key))
		{
			#ifdef DEBUG
			printf("InHash(): found key '%s' at position %ld.\n",key,entry->n);
			#endif
			return entry->n;
		}
		entry=entry->next;
	}

	#ifdef DEBUG
	printf("InHash(): key '%s' is not found.\n",key);
	#endif

	return -1;
}

void* GetHashEntry(HASH_PTR hash,char* key)
{
	long n;

	#ifdef DEBUG
	printf("GetHashEntry(): key is '%s'.\n",key);
	#endif

	n=InHash(hash,key);
	if( n == -1 )
		return NULL;

	#ifdef DEBUG
	printf("GetHashEntry(): element pointer is %ld.\n",
	(long)(hash->elements[n]));
	#endif

	return hash->elements[n];
}

int ToHash(HASH_PTR hash,char* key,void* element)
{
	long hval,len;
	HASH_ENTRY_PTR* pentry;
	HASH_ENTRY_PTR entry;
	char* ptr;

	if( InHash(hash,key) >= 0 )
	/* Already in hash. */
		return 0;

	if( hash->free >= hash->size )
		return -1;

	if( !(entry=(HASH_ENTRY_PTR)malloc(sizeof(HASH_ENTRY))) )
		return -1;

	len = strlen(key);
	if( !(ptr=(char*)malloc((len+1)*sizeof(char))) )
	{
		free(entry);
		return -1;
	}
	strlcpy(ptr, key, len + 1);

	hval=__HashFunc(key,hash->size);
	pentry=hash->hentry+hval;
	while( *pentry )
		pentry=&((*pentry)->next);

	*pentry=entry;
	entry->key=ptr;
	entry->n=hash->free;
	hash->elements[entry->n]=element;
	entry->next=NULL;
	hash->free++;

	#ifdef DEBUG
	printf("Added element with key '%s', index is %ld.\n",key,entry->n);
	#endif

	return 0;
}
