/*
 * $Id: ipc.c 13 2007-10-04 14:54:56Z rea $
 *
 * Copyright 2003-2007 Eygene A. Ryabinkin. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY EYGENE A. RYABINKIN "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EYGENE A. RYABINKIN OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#if !defined(lint)
static const char vcsid[] = "$Id: ipc.c 13 2007-10-04 14:54:56Z rea $";
#endif

#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <netinet/in.h>
#include <netdb.h>
#include <errno.h>
#include <string.h>
#include <strings.h>
#include "my.h"

int openUnixSocket(char* File)
/******************************************************************************
 * Opens UNIX domain socket.                                                  *
 * Call:                                                                      *
 *  File - socket filename.                                                   *
 * Return:                                                                    *
 *  Socket descriptor if no error occured, otherwise -1.                      *
 ******************************************************************************/
{
	int s;
	struct sockaddr_un addr;

	if( (s=socket(PF_LOCAL,SOCK_STREAM,0)) < 0 )
	{
		LogError(-1,"socket(): %s",strerror(errno));
		return -1;
	}

	LogDebug(10,"Obtained socket, descriptor %d",s);

	bzero((void*)&addr,sizeof(addr));
	addr.sun_family=PF_LOCAL;
	strlcpy(addr.sun_path, File, sizeof(addr.sun_path));

	if( connect(s,(struct sockaddr*)&addr,SUN_LEN(&addr)) < 0 )
	{
		close(s);
		LogError(-1,"connect(): %s",strerror(errno));
		return -1;
	}

	LogDebug(10,"UNIX domain socket %d connected.",s);

	return s;
}

int openTCPSocket(char* Host,int Port)
/******************************************************************************
 * Opens TCP stream.                                                          *
 * Call:                                                                      *
 *  Host - host name.                                                         *
 *  Port - foreign port.                                                      *
 * Return:                                                                    *
 *  Socket descriptor if no error occured, otherwise -1.                      *
 ******************************************************************************/
{
	struct hostent* he;
	struct sockaddr_in addr;
	int s;

	if( (s=socket(PF_INET,SOCK_STREAM,0)) < 0 )
	{
		LogError(-1,"socket(): %s",strerror(errno));
		return -1;
	}

	LogDebug(10,"Obtained socket, descriptor %d",s);

	addr.sin_family=PF_INET;
	addr.sin_port=htons(Port);
	he=gethostbyname(Host);
	if( !he || he->h_addrtype != AF_INET )
	{
		close(s);
		LogError(-1,"gethostbyname(): %s",strerror(errno));
		return -1;
	}
	memcpy(&addr.sin_addr.s_addr,he->h_addr_list[0],
	       sizeof(addr.sin_addr.s_addr));
	if( connect(s,(struct sockaddr*)&addr,sizeof(addr)) < 0 )
	{
		close(s);
		LogError(-1,"connect(): %s",strerror(errno));
		return -1;
	}

	LogDebug(10,"TCP socket %d connected.",s);

	return s;
}
