/*
 * $Id: kasper.c 13 2007-10-04 14:54:56Z rea $
 *
 * Copyright 2003-2007 Eygene A. Ryabinkin. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY EYGENE A. RYABINKIN "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EYGENE A. RYABINKIN OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#if !defined(lint)
static const char vcsid[] = "$Id: kasper.c 13 2007-10-04 14:54:56Z rea $";
#endif

#include <unistd.h>
#include <string.h>
#include <sys/param.h>
#include <time.h>
#include "my.h"
#include "kasper.h"

#define BUFLEN  (MAXPATHLEN+100)

int checkFileKAV(int Sock,char* Path)
{
	char req[BUFLEN];
	char clock[27],date[16];
	int time_pattern[]={8,9,10,4,5,6,7,11,12,13,14,15,16,17,18};
	long i,Result,Action;
	time_t tm;

	if (strlen(Path) > MAXPATHLEN) {
		LogError(-1,"checkFileKAV(): path length too big for me.\n");
		return -1;
	}

	strlcpy(req, "<0>", sizeof(req));
	/* Constructing date and time in format 01 Mar 00:00:00. */
	tm=time(NULL);
	ctime_r(&tm,clock);
	for(i=0;i<sizeof(time_pattern)/sizeof(int);i++)
		date[i]=clock[time_pattern[i]];
	date[i]='\0';
	strlcat(req, date, sizeof(req));
	strlcat(req, ":", sizeof(req));
	#if 0
	strlcat(req, "\xFEI0|MD|*\xFE", sizeof(req));
	#endif
	strlcat(req, Path, sizeof(req));

	LogDebug(9,"KAV enquiry is '%s'.",req);

	write(Sock, req, strlen(req));
	read(Sock,&Result,2);
	Action=(Result&0xFFFF)>>8;
	Result=(Result&0xFF)-0x30;

	LogDebug(9,"KAV return code is %lXh, KAV action is %lXh.",Result+0x30,Action);

	return Result;
}

char* KAVError(long code)
{
	switch( code )
	{
		case 0:
			return "no viruses were found";
		case 1:
			return "virus scan was not complete";
		case 2:
			return "mutated or corrupted viruses were found";
		case 3:
			return "suspicious object(s) were found";
		case 4:
			return "infected by virus";
		case 5: /* Do not try to pass the cured messages. */
			return "infected by virus";
		case 6:
			return "infected by virus";
		case 7:
			return "corrupted object(s) were found";
		default:
			return "unknown KAV error";
	}

	/* Notreached. */
	return NULL;
}


#if 0
#endif
