/*
 * $Id: log_filter.c 13 2007-10-04 14:54:56Z rea $
 *
 * Copyright 2003-2007 Eygene A. Ryabinkin. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY EYGENE A. RYABINKIN "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EYGENE A. RYABINKIN OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#if !defined(lint)
static const char vcsid[] = "$Id: log_filter.c 13 2007-10-04 14:54:56Z rea $";
#endif

#include <sys/param.h>
#include <sys/file.h>
#include <sys/wait.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <syslog.h>
#include <signal.h>
#include <time.h>
#include <string.h>
#include <ctype.h>
#include "libmilter/mfapi.h"
#include "my.h"
#include "parse.h"
#include "stringops.h"

#define _DEF_DEF_PROTO  "local:"
#define _DEF_MY_SOCKET	"/var/run/.log_filter"
#define _DEF_MY_TEMPLATE "/tmp/.messXXXXXX"
#define _DEF_MY_IDENT   "log_filter"
#define MAILPROG        "/usr/sbin/sendmail"
#define DEFDIRMODE      0700
#define MAXDSIZE        1024
#define MAXMMAPSIZE     8*1024*1024

#define NUM_TOKENS (sizeof(tokens)/sizeof(TOKENS))
#define TOK_HOSTNAME    0
#define TOK_DATE        1
#define TOK_FROM        2
#define TOK_RCPT        3
#define TOK_SUBJECT     4
#define TOK_LEN         5
#define TOK_ACTION      6

char Hostname[MAXHOSTNAMELEN];
TOKENS tokens[]={
	{'H',Hostname},
	{'D',NULL},
	{'F',NULL},
	{'R',NULL},
	{'S',NULL},
	{'L',NULL},
	{'A',NULL}
};

typedef struct NAMELIST {
	char* name;
	void* next;
}NAMELIST,*NAMELIST_PTR;

typedef struct PRIV_DATA {
	int des;
	char name[MAXPATHLEN];
	long meslen;
	char* from;
	char* helohost;
	char* subject;
	NAMELIST root;
	NAMELIST_PTR last;
	TOKENS tokens[NUM_TOKENS];
	char* timedate[26];
	char len[20];
	long maxlen;
	int checkSize;
}PRIV_DATA,*PRIV_DATA_PTR;

char StripSymb[]="<>";
int Daemonize,PrintPid;
char temp_socket[MAXPATHLEN+sizeof(_DEF_DEF_PROTO)];
char DefFormat[]="%A %H %D: %F to %R with subject `%S', length %L bytes.\n";
char* FileTemplate=_DEF_MY_TEMPLATE;
char* Format;
char *Admin=NULL,*AdminMessage=NULL,*RejectMessage=NULL;
int LogFD,CheckSize,UseMMAP=1,LLevel=0;
HASH QuotasHash;
char *MailSpool=NULL,*CWFile=NULL;
char **LocalHostNames;
int LHNsize;

void SaveMessage(char*,char*,int,char*);
char** InitHostNames(int*,char*);

sfsistat mlfi_connect(SMFICTX* ctx,char* hostname,_SOCK_ADDR* hostaddr)
/******************************************************************************
 * Called for each SMTP connection.                                           *
 * Here we do memory allocation for this connection and some initialisation.  *
 ******************************************************************************/
{
	PRIV_DATA_PTR privData;
	long i;

	LogDebug(8,"*** New connection initiated. ***");
	if( !(privData=(void*)malloc(sizeof(PRIV_DATA))) )
	{
		LogError(-1,"%s,%d: unable to allocate memory",__FILE__,__LINE__);
		smfi_setpriv(ctx,NULL);
		return SMFIS_TEMPFAIL;
	}
	for(i=0;i<NUM_TOKENS;i++)
	{
		privData->tokens[i].name=tokens[i].name;
		privData->tokens[i].value=tokens[i].value;
	}
	privData->meslen=0;
	privData->helohost=NULL;
	privData->from=NULL;
	privData->subject=NULL;
	privData->maxlen=1024*1024*1024;
	privData->checkSize=0;
	if( MailSpool )
	/* Temporary file for message name. */
	{
		strlcpy(privData->name, FileTemplate, sizeof(privData->name));
		privData->des=mkstemp(privData->name);
		LogDebug(8,"Opened temporary file `%s', fd = %d.",
		privData->name,privData->des);
	}
	/* Set up recipients list. */
	privData->root.next=NULL;
	privData->last=&(privData->root);
	smfi_setpriv(ctx,privData);

	return SMFIS_CONTINUE;
}

sfsistat mlfi_helo(SMFICTX* ctx,char* helohost)
/******************************************************************************
 * Called for HELO/EHLO command issued.                                       *
 ******************************************************************************/
{
	PRIV_DATA_PTR privData;
	long len;

	privData=smfi_getpriv(ctx);
	len=strlen(helohost)+1;
	if( !(privData->helohost=(char*)malloc(len*sizeof(char))) )
	{
		LogError(-1,"Cannot allocate memory for helohost data.");
		return SMFIS_TEMPFAIL;
	}
	strlcpy(privData->helohost, helohost, len);
	return SMFIS_CONTINUE;
}

sfsistat mlfi_envfrom(SMFICTX* ctx,char** argv)
/******************************************************************************
 * FROM handler.                                                              *
 ******************************************************************************/
{
	PRIV_DATA_PTR privData;
	long len;

	privData=smfi_getpriv(ctx);
	len=strlen(argv[0])+1;
	if( !(privData->from=(char*)malloc(len*sizeof(char))) )
	{
		LogError(-1,"Cannot allocate memory for `FROM' field.");
		return SMFIS_TEMPFAIL;
	}
	strlcpy(privData->from, argv[0], len);
	StripSymbols(privData->from,StripSymb,sizeof(StripSymb)/sizeof(char)-1);
	len=0;
	while( privData->from[len] )
	{
		privData->from[len]=tolower(privData->from[len]);
		len++;
	}

	LogDebug(8,"MAIL FROM '%s'.",privData->from);

	return SMFIS_CONTINUE;
}

sfsistat mlfi_envrcpt(SMFICTX* ctx,char** argv)
/******************************************************************************
 * RCPT handler.                                                              *
 ******************************************************************************/
{
	PRIV_DATA_PTR privData;
	long len;

	privData=smfi_getpriv(ctx);
	if( !(privData->last->next=(NAMELIST_PTR)malloc(sizeof(NAMELIST))) )
	{
		LogError(-1,"Cannot allocate memory for next recipient.");
		return SMFIS_TEMPFAIL;
	}
	privData->last=privData->last->next;
	privData->last->next=NULL;
	privData->last->name=NULL;
	len=strlen(argv[0])+1;
	if( !(privData->last->name=(char*)malloc(len*sizeof(char))) )
	{
		LogError(-1,"Cannot allocate memory for recipient name.");
		return SMFIS_TEMPFAIL;
	}
	strlcpy(privData->last->name, argv[0], len);
	StripSymbols(privData->last->name,StripSymb,
	sizeof(StripSymb)/sizeof(char)-1);
	len=0;
	while( privData->last->name[len] )
	{
		privData->last->name[len]=tolower(privData->last->name[len]);
		len++;
	}

	if( CheckSize && InHash(&QuotasHash,privData->last->name) != -1 )
	{
		len=*((long*)GetHashEntry(&QuotasHash,privData->last->name));
		if( privData->maxlen > len )
			privData->maxlen=len;
		privData->checkSize=1;
	}

	LogDebug(8,"RCPT TO '%s'.",privData->last->name);

	return SMFIS_CONTINUE;
}

sfsistat mlfi_header(SMFICTX* ctx,char* name,char* value)
/******************************************************************************
 * Called for each header line.                                               *
 ******************************************************************************/
{
	PRIV_DATA_PTR privData;
	long len;

	if( (privData=smfi_getpriv(ctx)) == NULL )
	{
		LogError(-1,"%s,%d: private data is NULL.",__FILE__,__LINE__);
		return SMFIS_TEMPFAIL;
	}

	if( MailSpool )
	/* Adding this header to message file. */
	{
		write(privData->des, name, strlen(name));
		write(privData->des, ": ", 2);
		write(privData->des, value, strlen(value));
		write(privData->des, "\n", 1);
	}

	privData->meslen += strlen(name) + strlen(value) + 2;

	if (!strncmp(name, "Subject", 7))
	{
		len = strlen(value);
		if( !(privData->subject=(char*)calloc(len+1,sizeof(char))) )
		{
			LogError(-1,"%s: cannot allocate memory for `Subject' string.");
			return SMFIS_TEMPFAIL;
		}
		strlcpy(privData->subject, value, len + 1);
	}

	return SMFIS_CONTINUE;
}

sfsistat mlfi_eoh(SMFICTX* ctx)
/******************************************************************************
 * Called when all headers were processed.                                    *
 ******************************************************************************/
{
	PRIV_DATA_PTR privData;

	if( (privData=smfi_getpriv(ctx)) == NULL )
	{
		LogError(-1,"%s,%d: private data is NULL.",__FILE__,__LINE__);
		return SMFIS_TEMPFAIL;
	}

	if( MailSpool )
		write(privData->des,"\n",1);
	
	return SMFIS_CONTINUE;
}

sfsistat mlfi_body(SMFICTX* ctx,unsigned char* bodychunk,size_t len)
/******************************************************************************
 * Called for each body chunk.                                                *
 ******************************************************************************/
{
	PRIV_DATA_PTR privData;

	privData=smfi_getpriv(ctx);
	if( !privData )
	{
		LogError(-1,"%s,%d: NULL private data in mlfi_body()",__FILE__,__LINE__);
		return SMFIS_TEMPFAIL;
	}

	if( MailSpool )
		write(privData->des,bodychunk,len);

	privData->meslen+=len;

	return SMFIS_CONTINUE;
}

sfsistat mlfi_eom(SMFICTX* ctx)
/******************************************************************************
 * Called when message is completed. Last chance to make any modifications.   *
 ******************************************************************************/
{
	PRIV_DATA_PTR privData;
	NAMELIST_PTR ptr;
	int free_rcpts=0,reject;
	long len,i,fsize;
	char* rcpts,buf[MAXDSIZE],*mmdes=NULL;
	time_t tm;
	int des;
	long pid;
	struct stat sb;

	LogDebug(8,"%s,%d: mlfi_eom() called",__FILE__,__LINE__);
	privData=smfi_getpriv(ctx);
	if( !privData )
	{
		LogError(-1,"%s,%d: NULL private data in mlfi_eom()",__FILE__,__LINE__);
		return SMFIS_TEMPFAIL;
	}

	if( MailSpool )
	{
		/* Decide if we will mmap(2) this message. */
		fstat(privData->des,&sb);
		fsize=sb.st_size;
		if( UseMMAP && fsize < MAXMMAPSIZE )
		{
			LogDebug(8,"Mmapping message, size = %ld.",fsize);
			mmdes=mmap(NULL,fsize,PROT_READ,0,privData->des,0);
			LogDebug(10,"Fd = %d mapped to 0x%lx.",
			privData->des,(long)mmdes);
		}
		else
			LogDebug(8,"MessageFD = %d, size = %ld",
			privData->des,fsize);
		/* First we go through recipients list. */
		strlcpy(buf, MailSpool, sizeof(buf));
		strlcat(buf, "/In", sizeof(buf));
		ptr=(NAMELIST_PTR)(privData->root.next);
		while(ptr)
		{
			rcpts=ptr->name;
			while( *rcpts != '@' )
				rcpts++;
			rcpts++;
			for(i=0;i<LHNsize;i++)
				if (!strcmp(rcpts, LocalHostNames[i])) {
					SaveMessage(buf,ptr->name,privData->des,mmdes);
					break;
				}
			ptr=ptr->next;
		}
		/* Check sender. */
		strlcpy(buf, MailSpool, sizeof(buf));
		strlcat(buf, "/Out", sizeof(buf));
		rcpts=privData->from;
		while( *rcpts != '@' )
			rcpts++;
		rcpts++;
		for(i=0;i<LHNsize;i++)
			if (!strcmp(rcpts, LocalHostNames[i])) {
				SaveMessage(buf,privData->from,privData->des,mmdes);
				break;
			}
		if( mmdes )
			munmap(mmdes,fsize);
		close(privData->des);
		privData->des=-1;
		unlink(privData->name);
	}

	reject=0;
	if( privData->checkSize && privData->meslen > privData->maxlen )
		reject=1;

	flock(LogFD,LOCK_EX);
	len=0;
	ptr=(NAMELIST_PTR)(privData->root.next);
	while( ptr )
	{
		len += strlen(ptr->name) + 1;
		ptr=ptr->next;
	}
	if( len == 1 || !(rcpts=(char*)malloc((len+1)*sizeof(char))) )
	{
		free_rcpts=0;
		rcpts="some recipients";
	}
	else
	{
		free_rcpts=1;
		rcpts[0]='\0';
		ptr=(NAMELIST_PTR)(privData->root.next);
		while( ptr )
		{
			strlcat(rcpts, ptr->name, len + 1);
			strlcat(rcpts, ",", len + 1);
			ptr=ptr->next;
		}
		/* Kill last comma. */
		rcpts[len-1]='\0';
	}
	tm=time(NULL);
	ctime_r(&tm,(char*)(privData->timedate));
	chomp((char*)(privData->timedate));
	privData->tokens[TOK_DATE].value=(char*)(privData->timedate);
	privData->tokens[TOK_FROM].value=privData->from;
	privData->tokens[TOK_RCPT].value=rcpts;
	privData->tokens[TOK_SUBJECT].value=privData->subject;
	snprintf(privData->len,sizeof(privData->len),"%ld",privData->meslen);
	privData->tokens[TOK_LEN].value=(privData->len);
	if( reject )
		privData->tokens[TOK_ACTION].value="R";
	else
		privData->tokens[TOK_ACTION].value="A";
	my_printf(LogFD,Format,privData->tokens,NUM_TOKENS);
	if( free_rcpts )
		free(rcpts);
	flock(LogFD,LOCK_UN);

	if( reject )
	{
		if( Admin )
		/* Sending a warning to admin. */
		{
			des=SpawnMail(MAILPROG,&pid,Admin);
			my_printf(des,AdminMessage,privData->tokens,NUM_TOKENS);
			/* Terminator. */
			write(des,"\n.\n",3);
			close(des);
			waitpid(pid,&des,0);
		}
		if( RejectMessage )
		/* Sending a warning to sender. */
		{
			des=SpawnMail(MAILPROG,&pid,privData->from);
			my_printf(des,RejectMessage,privData->tokens,NUM_TOKENS);
			/* Terminator. */
			write(des,"\n.\n",3);
			close(des);
			waitpid(pid,&des,0);
		}
		return SMFIS_DISCARD;
	}

	return SMFIS_CONTINUE;
}

sfsistat mlfi_abort(SMFICTX* ctx)
/******************************************************************************
 * Called when message being aborted.                                         *
 ******************************************************************************/
{
	return SMFIS_CONTINUE;
}

sfsistat mlfi_close(SMFICTX* ctx)
/******************************************************************************
 * The very last handler.                                                     *
 ******************************************************************************/
{
	PRIV_DATA_PTR privData;

	NAMELIST_PTR ptr,temp;
	privData=smfi_getpriv(ctx);
	if( !privData )
		return SMFIS_CONTINUE;

	if( MailSpool )
	{
		if( privData->des != -1 )
			close(privData->des);
		unlink(privData->name);
	}
	ptr=(NAMELIST_PTR)(privData->root.next);
	while(ptr)
	{
		if( ptr->name )
			free(ptr->name);
		temp=ptr;
		ptr=temp->next;
		free(temp);
	}

	if( privData->from )
		free(privData->from);
	if( privData->helohost )
		free(privData->helohost);
	if( privData->subject )
		free(privData->subject);
	if( privData )
		free(privData);
	smfi_setpriv(ctx,NULL);
	LogDebug(8,"*** Data freed, connection closed. ***");

	return SMFIS_CONTINUE;
}

struct smfiDesc smfilter = {
	"LogMail",           /* Filter name. */
	SMFI_VERSION,        /* Version. */
	0,                   /* Flags. */
	mlfi_connect,        /* Connection handler. */
	mlfi_helo,           /* HELO command handler. */
	mlfi_envfrom,        /* Sender envelope handler. */
	mlfi_envrcpt,        /* Recipient envelope handler. */
	mlfi_header,         /* Header handler. */
	mlfi_eoh,            /* End-of-header handler. */
	mlfi_body,           /* Body handler. */
	mlfi_eom,            /* End-of-message handler. */
	mlfi_abort,          /* Message abort handler. */
	mlfi_close           /* Cleanup handler. */
};

void parseCmdLine(int,char*[]);
int parse_quotas(char*,HASH_PTR);
int main(int argc,char* argv[])
{
	int pid;

	parseCmdLine(argc,argv);
	LocalHostNames=InitHostNames(&LHNsize,CWFile);
	gethostname(Hostname, sizeof(Hostname));
	if (!Hostname[0])
		strlcpy(Hostname, "Amnesiac", sizeof(Hostname));

	if( PrintPid )
		printf("%ld\n",(long)getpid());

	if( Daemonize )
	{
		while( (pid=fork()) == -1 )
			sleep(5);
		if( pid )
			exit(0);
		close(0); close(1); close(2);
		LogOpenSyslog(_DEF_MY_IDENT,LOG_MAIL);
	}
	else
	{
		LogOpenStderr();
	}
	SetLogLevel(LLevel);

	unlink(_DEF_MY_SOCKET);
	strlcpy(temp_socket, _DEF_DEF_PROTO, sizeof(temp_socket));
	strlcat(temp_socket, _DEF_MY_SOCKET, sizeof(temp_socket));
	if( smfi_setconn(temp_socket) == MI_FAILURE )
	{
		LogError(-1,"smfi_setconn(\'%s\') failed",_DEF_MY_SOCKET);
		return -1;
	}
	if( smfi_register(smfilter) == MI_FAILURE )
	{
		LogError(-1,"Cannot register filter");
		return -1;
	}

	LogDebug(8,"Starting main loop.");
	return smfi_main();
}

void parseCmdLine(int argc,char* argv[])
{
	char result;
	int MessageFD;
	long size,i;
	struct stat sb;
	char dir[MAXDSIZE],*subdir[]={"/In","/Out"};

	Daemonize=0;
	PrintPid=0;
	LogFD=-1;
	CheckSize=0;
	UseMMAP=1;
	Format=(char*)DefFormat;

	while( (result=getopt(argc,argv,"dpmR:F:Q:A:M:L:S:H:v:")) != -1 )
	{
		if( result == 'd' )
			Daemonize=1;
		else if( result == 'p' )
			PrintPid=1;
		else if( result == 'm' )
			UseMMAP=0;
		else if( result == 'v' )
			LLevel=atoi(optarg);
		else if( result == 'H' )
		{
			size = strlen(optarg) + 1;
			if (!(CWFile=(char*)calloc(size, sizeof(char)))) {
				printf("Unable to allocate memory for CW filename.\n");
				exit(-1);
			}
			strlcpy(CWFile, optarg, size);
		}
		else if( result == 'S' )
		{
			size = strlen(optarg) + 1;
			if (!(MailSpool=(char*)calloc(size, sizeof(char)))) {
				printf("Unable to allocate memory for messages spool directory.\n");
				exit(-1);
			}
			strlcpy(MailSpool, optarg, size);
			/* Perform some checks. */
			i=0;
			while(MailSpool[i])
				i++;
			if( MailSpool[i-1] == '/' )
				MailSpool[i-1]='\0';
			if( stat(MailSpool,&sb) )
			{
				printf("Can't stat `%s': %s.\n",MailSpool,strerror(errno));
				exit(-1);
			}
			if( !(sb.st_mode&S_IFDIR) )
			{
				printf("Mail spool `%s' is not a directory.\n",MailSpool);
				exit(-1);
			}
			if( (sb.st_mode&0777) != DEFDIRMODE )
			{
				printf("Mail spool directory `%s' must be mode %o.\n",MailSpool,
				DEFDIRMODE);
				exit(-1);
			}
			/* Create subdirectories `In' and `Out' if they aren't exists. */
			for(i=0;i<(sizeof(subdir)/sizeof(char*));i++)
			{
				strlcpy(dir, MailSpool, sizeof(dir));
				strlcat(dir, subdir[i], sizeof(dir));
				if( stat(dir,&sb) )
				/* Creating. */
				{
					if( mkdir(dir,DEFDIRMODE) )
					{
						printf("Unable to create directory `%s': %s.\n",dir,
						strerror(errno));
						exit(-1);
					}
				}
				else
				/* Checking permissions. */
				{
					if( !(sb.st_mode&S_IFDIR) )
					{
						printf("`%s' isn't a directory as it ought to be.\n",dir);
						exit(-1);
					}
					if( (sb.st_mode&0777) != DEFDIRMODE )
					{
						printf("Directory `%s' must be mode %o.\n",MailSpool,DEFDIRMODE);
						exit(-1);
					}
				}
			}
		}
		else if( result == 'L' )
		{
			size = strlen(optarg) + 2;
			if (!(Format=(char*)calloc(size, sizeof(char)))) {
				printf("Unable to allocate memory for string template `%s'.\n",optarg);
				exit(-1);
			}
			strlcpy(Format, optarg, size);
			/* Check, if '\n' is missing at the end of the line. */
			i=0;
			while(Format[i])
				i++;
			if( Format[i-1] != '\n' )
			{
				Format[i]='\n';
				Format[i+1]='\0';
			}
		}
		else if( result == 'R' )
		{
			if( (MessageFD=open(optarg,O_RDONLY)) < 0 )
			{
				printf("Unable to open reject message template `%s'.\n",
				optarg);
				exit(-1);
			}
			size=lseek(MessageFD,0,SEEK_END);
			lseek(MessageFD,0,SEEK_SET);
			if( !(RejectMessage=(char*)calloc(size+1,sizeof(char))) )
			{
				printf("Unable to allocate memory for reject message template.\n");
				exit(-1);
			}
			size=read(MessageFD,RejectMessage,size);
			RejectMessage[size+1]='\0';
			close(MessageFD);
		}
		else if( result == 'F' )
		{
			if( (LogFD=open(optarg,O_WRONLY|O_CREAT|O_APPEND,0600)) < 0 )
			{
				printf("Unable to open log file `%s'.\n",optarg);
				exit(-1);
			}
		}
		else if( result == 'Q' )
		{
			CheckSize=1;
			if( !parse_quotas(optarg,&QuotasHash) )
				exit(-1);
		}
		else if( result == 'A' )
		{
			size = strlen(optarg) + 1;
			if (!(Admin=(char*)calloc(size, sizeof(char)))) {
				printf("Unable to allocate memory for administrator mail `%s'.\n",
				optarg);
				exit(-1);
			}
			strlcpy(Admin, optarg, size);
		}
		else if( result == 'M' )
		{
			if( (MessageFD=open(optarg,O_RDONLY)) < 0 )
			{
				printf("Unable to open administrator message template `%s'.\n",
				optarg);
				exit(-1);
			}
			size=lseek(MessageFD,0,SEEK_END);
			lseek(MessageFD,0,SEEK_SET);
			if( !(AdminMessage=(char*)calloc(size+1,sizeof(char))) )
			{
				printf("Unable to allocate memory for administrator message template.\n");
				exit(-1);
			}
			size=read(MessageFD,AdminMessage,size);
			AdminMessage[size+1]='\0';
			close(MessageFD);
		}
	}

	if( LogFD == -1 )
	{
		printf("Log file must be specified with -F option.\n");
		exit(-1);
	}

	return;
}

struct quota {
	char* name;
	long* size;
	struct quota* next;
};

#define BUFLEN 1024

int parse_quotas(char* name,HASH_PTR hash)
/******************************************************************************
 * Parses file with user quotas and hashes it.                                *
 ******************************************************************************/
{
	char buffer[BUFLEN];
	long len,i,size;
	struct quota *root,*cur,*temp;
	FILE* fp;

	if( !(fp=fopen(name,"r")) )
	{
		printf("Unable to open quotas file `%s'.\n",name);
		return 0;
	}
	root=temp=NULL;
	size=0;
	fgets(buffer,BUFLEN,fp);
	while( !feof(fp) )
	{
		chomp(buffer);
		i=0;
		while( buffer[i] == '\t' || buffer[i] == ' ' )
			i++;
		if( buffer[i] != '#' && buffer[i] != '\0' )
		{
			len=0;
			while( buffer[len+i] != '\t' && buffer[len+i] != ' ' )
				len++;
			if (!(cur=
			    (struct quota *)malloc(sizeof(struct quota))) ||
			    !(cur->size=malloc(sizeof(long))) ||
			    !(cur->name=calloc(len + 1, sizeof(char)))) {
				printf("Unable to allocate memory.\n");
				return 0;
			}
			if( !root )
				root=cur;
			if( temp )
				temp->next=cur;
			cur->next=NULL;
			size++;
			strlcpy(cur->name, buffer+i, len + 1);
			cur->name[len+1]='\0';
			sscanf(buffer+i+len+1,"%ld",cur->size);
			temp=cur;
			cur=cur->next;
		}
		fgets(buffer,BUFLEN,fp);
	}

	/* OK(?), let's create hash. */
	if( CreateHash(hash,size) == -1 )
	{
		printf("Cannot create hash for quotas.\n");
		return 0;
	}

	cur=root;
	while( cur )
	{
		ToHash(hash,cur->name,cur->size);
		temp=cur;
		cur=cur->next;
		free(temp->name);
		free(temp);
	}
	
	return 1;
}

#define WRBUFSIZE  16*1024
void SaveMessage(char* spool,char* user,int fdes,char* p)
/******************************************************************************
 * Saves message to some file located in spool/user/ directory.               *
 *  If p != NULL message file is assumed to be mmap'ped and referenced by p.  *
 * This case is generally better from perfomance point of view, but takes     *
 * extra memory.                                                              *
 * Remark: fdes must be valid in any case.                                    *
 ******************************************************************************/
{
	struct stat sb;
	char filename[MAXPATHLEN],filectl[MAXPATHLEN],name[MAXPATHLEN];
	char buf[WRBUFSIZE],*ptr;
	long fnum,next,size,filesize;
	int makedir=0,noctl=0,des,tm_mday,tm_mon,tm_year;
	struct tm ltime;
	time_t tm;

	if ((strlen(spool) + strlen(user) + 30) > MAXPATHLEN) {
		LogError(-1,"Cannot save message: path too long.");
		return;
	}
	if( fstat(fdes,&sb) )
	{
		LogError(-1,"Can't stat(des=%d): %s.",fdes,strerror(errno));
		return;
	}
	filesize=sb.st_size;
	tm = time(NULL);
	localtime_r(&tm, &ltime);

	strlcpy(filename, spool, sizeof(filename));
	strlcat(filename, "/", sizeof(filename));
	strlcat(filename, user, sizeof(filename));
	strlcpy(filectl, filename, sizeof(filectl));
	strlcat(filectl, "/.ctl", sizeof(filectl));
	LogDebug(8,"Dir `%s', ctl = `%s', message size = %ld.",
	filename,filectl,filesize);
	if( stat(filename,&sb) )
	{
		makedir=1;
		fnum=0;
	}
	else if( !(sb.st_mode&S_IFDIR) )
	{
		makedir=1;
		fnum=0;
		if( unlink(filename) )
		{
			LogError(-1,"Unable to unlink `%s': %s.",filename,strerror(errno));
			return;
		}
	}

	if( makedir )
	{
		if( mkdir(filename,DEFDIRMODE) )
		{
			LogError(-1,"Cannot create directory `%s': %s.\n",
			filename,strerror(errno));
			return;
		}
		noctl=1;
		LogDebug(8,"Created directory `%s'.",filename);
	}

	if( !noctl && !stat(filectl,&sb) && (sb.st_mode & S_IFREG) )
	/* File exists. Open for r/w. */
	{
		if( (des=open(filectl,O_RDWR|O_EXLOCK)) < 0 )
		{
			LogError(-1,"Cannot open control file `%s': %s.",
			filectl,strerror(errno));
			return;
		}
		read(des,&fnum,sizeof(fnum));
		read(des,&tm_mday,sizeof(tm_mday));
		read(des,&tm_mon,sizeof(tm_mon));
		read(des,&tm_year,sizeof(tm_year));
		if( tm_mday == ltime.tm_mday && tm_mon == ltime.tm_mon &&
				tm_year == ltime.tm_year )
		{
			next=fnum+1;
			LogDebug(8,"Next message number is %ld.",next);
		}
		else
		{
			fnum=0;
			next=1;
			tm_mday=ltime.tm_mday;
			tm_mon=ltime.tm_mon;
			tm_year=ltime.tm_year;
			LogDebug(8,"Day changed. Next message number is %ld.",next);
		}
		lseek(des,0,SEEK_SET);
		write(des,&next,sizeof(next));
		write(des,&tm_mday,sizeof(tm_mday));
		write(des,&tm_mon,sizeof(tm_mon));
		write(des,&tm_year,sizeof(tm_year));
		flock(des,LOCK_UN);
		close(des);
		LogDebug(8,"Control file updated.");
	}
	else
	/* No control file. */
	{
		fnum=0;
		next=1;
		tm_mday=ltime.tm_mday;
		tm_mon=ltime.tm_mon;
		tm_year=ltime.tm_year;
		unlink(filectl);
		if( (des=open(filectl,O_WRONLY|O_CREAT|O_EXLOCK,0600)) < 0 )
		{
			LogError(-1,"Cannot create control file `%s': %s.",
			filectl,strerror(errno));
			return;
		}
		write(des,&next,sizeof(next));
		write(des,&tm_mday,sizeof(tm_mday));
		write(des,&tm_mon,sizeof(tm_mon));
		write(des,&tm_year,sizeof(tm_year));
		flock(des,LOCK_UN);
		close(des);
		LogDebug(8,"Control file created.");
	}

	snprintf(name,sizeof(name),"/%02d.%02d.%04d_%ld",tm_mday,tm_mon+1,
	tm_year+1900,fnum);
	strlcat(filename, name, sizeof(filename));
	LogDebug(8,"Message file `%s'.",filename);
	
	if( (des=open(filename,O_WRONLY|O_CREAT|O_TRUNC,0600)) < 0 )
	{
		LogError(-1,"Unable to create file `%s': %s.\n",filename,
		strerror(errno));
		return;
	}

	if( p )
	/* Mmap'ped file. */
	{
		ptr=p;
		size=filesize;
		while( filesize && size )
		{
			size=write(des,ptr,filesize);
			filesize-=size;
			ptr+=size;
		}
	}
	else
	{
		lseek(fdes,0,SEEK_SET);
		while( (size=read(fdes,buf,WRBUFSIZE)) > 0 )
		{
			LogDebug(10,"Writing %ld bytes.",size);
			write(des,buf,size);
		}
	}
	close(des);

	return;
}

char** InitHostNames(int* arrsize,char* file)
/******************************************************************************
 * Initialysing a list of host names for which we will mirror mail.           *
 ******************************************************************************/
{
	char buf[MAXHOSTNAMELEN],**names;
	FILE* fp;
	NAMELIST root,*cur,*ptr;
	long i, size, len;

	if( file == NULL )
	/* Only hostname. */
	{
		gethostname(buf, sizeof(buf));
		*arrsize=1;
		len = strlen(buf) + 1;
		if (!(names = (char**)calloc(1, sizeof(char*))) ||
		    !(names[0] = (char*)calloc(len, sizeof(char)))) {
			printf("Unable to allocate memory.\n");
			exit(-1);
		}
		strlcpy(names[0], buf, len);
	}
	else
	{
		gethostname(buf, sizeof(buf));
		*arrsize=1;
		len = strlen(buf) + 1;
		if (!(root.name=(char*)calloc(len, sizeof(char)))) {
			printf("Unable to allocate memory.\n");
			exit(-1);
		}
		strlcpy(root.name, buf, len);
		root.next=NULL;
		cur=&root;
		if( !(fp=fopen(file,"r")) )
		{
			printf("Unable to open file `%s' with host aliases.\n",file);
			exit(-2);
		}
		fgets(buf,MAXHOSTNAMELEN,fp);
		while( !feof(fp) )
		{
			chomp(buf);
			i=0; size=0;
			while( buf[i] == ' ' && buf[i] == '\t' )
				if( buf[i] == '\0' )
				{
					size=-1;
					break;
				}
				else
					i++;
			if( size != -1 )
			{
				size=0;
				while( buf[i+size] != ' ' && buf[i+size] != '\t' &&
							 buf[i+size] != '#' && buf[i+size] != '\0' )
					size++;
				if( size )
				{
					if (!(ptr=(NAMELIST_PTR)malloc(sizeof(NAMELIST))) ||
					    !(ptr->name=(char*)calloc(size+1, sizeof(char)))) {
						printf("Unable to allocate memory.\n");
						exit(-1);
					}
					strlcpy(ptr->name, buf + i, size + 1);
					(*arrsize)++;
					ptr->name[size]='\0';
					ptr->next=NULL;
					cur->next=ptr;
					cur=ptr;
				}
			}
			fgets(buf,MAXHOSTNAMELEN,fp);
		}
		if( !(names=(char**)calloc(*arrsize,sizeof(char*))) )
		{
			printf("Unable to allocate memory.\n");
			exit(-1);
		}
		cur=&root; i=0;
		while( cur )
		{
			printf("Host alias is `%s'.\n",cur->name);
			names[i]=cur->name;
			ptr=cur->next;
			if( cur != &root )
				free(cur);
			cur=ptr;
			i++;
		}
	}

	return names;
}
