/*
 * $Id: m_alloc.c 13 2007-10-04 14:54:56Z rea $
 *
 * Copyright 2003-2007 Eygene A. Ryabinkin. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY EYGENE A. RYABINKIN "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EYGENE A. RYABINKIN OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#if !defined(lint)
static const char vcsid[] = "$Id: m_alloc.c 13 2007-10-04 14:54:56Z rea $";
#endif

#include <stdlib.h>
#include <stdio.h>
#define INTERNALS
#include "m_alloc.h"
#undef INTERNALS

#ifndef NULL
#define NULL (void*)0
#endif

#if 0
#define DEBUG
#endif

long m_Size;
long m_Blocks;

void m_init(void)
{
	m_Size=0;
	m_Blocks=0;

	return;
}

void m_shutdown(void)
{
	if( m_Size != 0 )
	{
		printf("%s,%d: m_shutdown(): detected %ld bytes of unfreed memory.\n",__FILE__,__LINE__,m_Size);
	}
	if( m_Blocks != 0 )
	{
		printf("%s,%d: m_shutdown(): detected %ld unfreed memory blocks.\n",__FILE__,__LINE__,m_Blocks);
	}

	return;
}

void* m_malloc(long size)
/******************************************************************************
 * Allocates size bytes of memory and registers this block. Also fills this   *
 * block with zeros.                                                          *
 ******************************************************************************/
{
	void* ptr;
	M_MEMBLK_PTR blk;
	long i;
	char* data;

	#ifdef DEBUG
	printf("Allocating %ld Mb of memory.\n",size/1024/1024);
	#endif
	if( (ptr=malloc(size+sizeof(M_MEMBLK))) == NULL )
	{
		printf("%s,%d: unable to allocate %ld bytes of memory.\n",__FILE__,__LINE__,size);
		return NULL;
	}
	blk=(M_MEMBLK_PTR)ptr;
	blk->size=size;
	blk->magic=_M_MAGIC;
	m_Size+=size;
	m_Blocks++;
	ptr+=sizeof(M_MEMBLK);
	/* Cleaning and testing just allocated memory. */
	data=(char*)ptr;
	for(i=0;i<size;i++)
		data[i]='\0';

	return ptr;
}

void m_free(void* ptr)
{
	M_MEMBLK_PTR blk;

	ptr-=sizeof(M_MEMBLK);
	blk=(M_MEMBLK_PTR)ptr;
	if( blk->magic != 0x12345678 )
		printf("%s,%d: m_free(): magic corrupted, block #%ld.\n",__FILE__,__LINE__,m_Blocks);
	m_Size-=blk->size;
	m_Blocks--;
	if( m_Size < 0 || m_Blocks < 0 )
		printf("%s,%d: m_free(): freeing too much memory, possibly corrupted heap, block #%ld.\n",__FILE__,__LINE__,m_Blocks+1);
	free(ptr);

	return;
}

void m_info(void)
{
	printf("m_info(): allocated %ld byte(s) of memory in %ld block(s).\n",
	m_Size,m_Blocks);

	return;
}
