/*
 * $Id: mail_filter.c 13 2007-10-04 14:54:56Z rea $
 *
 * Copyright 2003-2007 Eygene A. Ryabinkin. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY EYGENE A. RYABINKIN "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EYGENE A. RYABINKIN OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#if !defined(lint)
static const char vcsid[] = "$Id: mail_filter.c 13 2007-10-04 14:54:56Z rea $";
#endif

#include <sys/param.h>
#include <sys/file.h>
#include <sys/wait.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <syslog.h>
#include <signal.h>
#include <time.h>
#include "libmilter/mfapi.h"
#include "my.h"
#include "parse.h"
#include "rules.h"

#define DEBUG

#define _DEF_DEF_PROTO  "local:"
#define _DEF_MY_SOCKET	"/var/run/.mail_filter"
#define _DEF_MY_IDENT   "mail_filter"
#define MAILPROG        "/usr/sbin/sendmail"

#define NUM_TOKENS (sizeof(tokens)/sizeof(TOKENS))
#define TOK_HOSTNAME    0
#define TOK_DATE        1
#define TOK_FROM        2
#define TOK_RCPT        3
#define TOK_SUBJECT     4
#define TOK_LEN         5
#define TOK_ACTION      6

char Hostname[MAXHOSTNAMELEN];
TOKENS tokens[]={
	{'H',Hostname},
	{'D',NULL},
	{'F',NULL},
	{'R',NULL},
	{'S',NULL},
	{'L',NULL},
	{'A',NULL}
};

typedef struct NAMELIST {
	char* name;
	void* next;
}NAMELIST,*NAMELIST_PTR;

typedef struct PRIV_DATA {
	long meslen;
	char* from;
	char* helohost;
	char* subject;
	NAMELIST root;
	NAMELIST_PTR last;
	TOKENS tokens[NUM_TOKENS];
	char* timedate[26];
	char len[20];
	long maxlen;
	int checkSize;
}PRIV_DATA,*PRIV_DATA_PTR;

char StripSymb[]="<>";
int Daemonize,PrintPid;
char temp_socket[MAXPATHLEN+sizeof(_DEF_DEF_PROTO)];
int LogFD,CheckSize,LLevel=0;
RULE_PTR myRules;
int numRules,defaultRule;

int ruleApplies(RULE_PTR,PRIV_DATA_PTR);

sfsistat mlfi_connect(SMFICTX* ctx,char* hostname,_SOCK_ADDR* hostaddr)
/******************************************************************************
 * Called for each SMTP connection.                                           *
 * Here we do memory allocation for this connection and some initialisation.  *
 ******************************************************************************/
{
	PRIV_DATA_PTR privData;
	long i;

	LogDebug(8,"*** New connection initiated. ***");
	if( !(privData=(void*)malloc(sizeof(PRIV_DATA))) )
	{
		LogError(-1,"%s,%d: unable to allocate memory",__FILE__,__LINE__);
		smfi_setpriv(ctx,NULL);
		return SMFIS_TEMPFAIL;
	}
	for(i=0;i<NUM_TOKENS;i++)
	{
		privData->tokens[i].name=tokens[i].name;
		privData->tokens[i].value=tokens[i].value;
	}
	privData->meslen=0;
	privData->helohost=NULL;
	privData->from=NULL;
	privData->subject=NULL;
	privData->maxlen=1024*1024*1024;
	privData->checkSize=0;
	/* Set up recipients list. */
	privData->root.next=NULL;
	privData->last=&(privData->root);
	smfi_setpriv(ctx,privData);

	return SMFIS_CONTINUE;
}

sfsistat mlfi_helo(SMFICTX* ctx,char* helohost)
/******************************************************************************
 * Called for HELO/EHLO command issued.                                       *
 ******************************************************************************/
{
	PRIV_DATA_PTR privData;
	long len;

	privData=smfi_getpriv(ctx);
	len=strlen(helohost)+1;
	if( !(privData->helohost=(char*)malloc(len*sizeof(char))) )
	{
		LogError(-1,"Cannot allocate memory for helohost data.");
		return SMFIS_TEMPFAIL;
	}
	strlcpy(privData->helohost, helohost, len);

	LogDebug(8,"HELO `%s'.",privData->helohost);

	return SMFIS_CONTINUE;
}

sfsistat mlfi_envfrom(SMFICTX* ctx,char** argv)
/******************************************************************************
 * FROM handler.                                                              *
 ******************************************************************************/
{
	PRIV_DATA_PTR privData;
	long len;

	privData=smfi_getpriv(ctx);
	len=strlen(argv[0])+1;
	if( !(privData->from=(char*)malloc(len*sizeof(char))) )
	{
		LogError(-1,"Cannot allocate memory for from field.");
		return SMFIS_TEMPFAIL;
	}
	strlcpy(privData->from, argv[0], len);
	StripSymbols(privData->from,StripSymb,sizeof(StripSymb)/sizeof(char)-1);

	LogDebug(8,"MAIL FROM '%s'.",privData->from);

	return SMFIS_CONTINUE;
}

sfsistat mlfi_envrcpt(SMFICTX* ctx,char** argv)
/******************************************************************************
 * RCPT handler.                                                              *
 ******************************************************************************/
{
	PRIV_DATA_PTR privData;
	long len;

	privData=smfi_getpriv(ctx);
	if( !(privData->last->next=(NAMELIST_PTR)malloc(sizeof(NAMELIST))) )
	{
		LogError(-1,"Cannot allocate memory for next recipient.");
		return SMFIS_TEMPFAIL;
	}
	privData->last=privData->last->next;
	privData->last->next=NULL;
	privData->last->name=NULL;
	len=strlen(argv[0])+1;
	if( !(privData->last->name=(char*)malloc(len*sizeof(char))) )
	{
		LogError(-1,"Cannot allocate memory for recipient name.");
		return SMFIS_TEMPFAIL;
	}
	strlcpy(privData->last->name, argv[0], len);
	StripSymbols(privData->last->name,StripSymb,
	sizeof(StripSymb)/sizeof(char)-1);

	LogDebug(8,"RCPT TO '%s'.",privData->last->name);

	return SMFIS_CONTINUE;
}

sfsistat mlfi_header(SMFICTX* ctx,char* name,char* value)
/******************************************************************************
 * Called for each header line.                                               *
 ******************************************************************************/
{
	PRIV_DATA_PTR privData;
	long len;

	if( (privData=smfi_getpriv(ctx)) == NULL )
	{
		LogError(-1,"%s,%d: private data is NULL.",__FILE__,__LINE__);
		return SMFIS_TEMPFAIL;
	}

	privData->meslen += strlen(name) + strlen(value) + 2;

	if (!strncmp(name, "Subject", 7)) {
		len = strlen(value);
		if( !(privData->subject=(char*)calloc(len+1,sizeof(char))) )
		{
			LogError(-1,"%s: cannot allocate memory for `Subject' string.");
			return SMFIS_TEMPFAIL;
		}
		strlcpy(privData->subject, value, len + 1);
		LogDebug(8,"Subject: `%s'.",privData->subject);
	}

	return SMFIS_CONTINUE;
}

sfsistat mlfi_eoh(SMFICTX* ctx)
/******************************************************************************
 * Called when all headers were processed.                                    *
 ******************************************************************************/
{
	return SMFIS_CONTINUE;
}

sfsistat mlfi_body(SMFICTX* ctx,unsigned char* bodychunk,size_t len)
/******************************************************************************
 * Called for each body chunk.                                                *
 ******************************************************************************/
{
	PRIV_DATA_PTR privData;

	privData=smfi_getpriv(ctx);
	if( !privData )
	{
		LogError(-1,"%s,%d: NULL private data in mlfi_body()",__FILE__,__LINE__);
		return SMFIS_TEMPFAIL;
	}

	privData->meslen+=len;

	return SMFIS_CONTINUE;
}

sfsistat mlfi_eom(SMFICTX* ctx)
/******************************************************************************
 * Called when message is completed. Last chance to make any modifications.   *
 ******************************************************************************/
{
	PRIV_DATA_PTR privData;
	int reject,i;
	RULE_PTR curRule;

	LogDebug(8,"%s,%d: mlfi_eom() called",__FILE__,__LINE__);
	privData=smfi_getpriv(ctx);
	if( !privData )
	{
		LogError(-1,"%s,%d: NULL private data in mlfi_eom()",__FILE__,__LINE__);
		return SMFIS_TEMPFAIL;
	}

	/* Checking letter against rules. */
	if( defaultRule == PASS_RULE )
		reject=0;
	else
		reject=1;
	for(i=0;i<numRules;i++)
	{
		LogDebug(8,"Checking against rule #%d.",i);
		curRule=myRules+i;
		if( ruleApplies(curRule,privData) )
		{
			LogDebug(8,"Rule applies.");
			if( curRule->rule_type == PASS_RULE )
			{
				LogDebug(8,"Passing the letter.");
				reject=0;
			}
			else if( curRule->rule_type == DROP_RULE )
			{
				LogDebug(8,"Dropping the letter.");
				reject=1;
			}
			i=numRules;
		}
	}

	if( reject )
	{
		return SMFIS_DISCARD;
	}

	return SMFIS_CONTINUE;
}

sfsistat mlfi_abort(SMFICTX* ctx)
/******************************************************************************
 * Called when message being aborted.                                         *
 ******************************************************************************/
{
	return SMFIS_CONTINUE;
}

sfsistat mlfi_close(SMFICTX* ctx)
/******************************************************************************
 * The very last handler.                                                     *
 ******************************************************************************/
{
	PRIV_DATA_PTR privData;

	NAMELIST_PTR ptr,temp;
	privData=smfi_getpriv(ctx);
	if( !privData )
		return SMFIS_CONTINUE;

	ptr=(NAMELIST_PTR)(privData->root.next);
	while(ptr)
	{
		if( ptr->name )
			free(ptr->name);
		temp=ptr;
		ptr=temp->next;
		free(temp);
	}

	if( privData->from )
		free(privData->from);
	if( privData->helohost )
		free(privData->helohost);
	if( privData->subject )
		free(privData->subject);
	if( privData )
		free(privData);
	smfi_setpriv(ctx,NULL);
	LogDebug(8,"*** Data freed, connection closed. ***");

	return SMFIS_CONTINUE;
}

struct smfiDesc smfilter = {
	"MailFilter",        /* Filter name. */
	SMFI_VERSION,        /* Version. */
	0,                   /* Flags. */
	mlfi_connect,        /* Connection handler. */
	mlfi_helo,           /* HELO command handler. */
	mlfi_envfrom,        /* Sender envelope handler. */
	mlfi_envrcpt,        /* Recipient envelope handler. */
	mlfi_header,         /* Header handler. */
	mlfi_eoh,            /* End-of-header handler. */
	mlfi_body,           /* Body handler. */
	mlfi_eom,            /* End-of-message handler. */
	mlfi_abort,          /* Message abort handler. */
	mlfi_close           /* Cleanup handler. */
};

void parseCmdLine(int,char*[]);
int main(int argc,char* argv[])
{
	int pid;

	parseCmdLine(argc,argv);
	gethostname(Hostname, sizeof(Hostname));
	if (!Hostname[0])
		strlcpy(Hostname, "Amnesiac", sizeof(Hostname));

	if( PrintPid )
		printf("%ld\n",(long)getpid());

	if( Daemonize )
	{
		while( (pid=fork()) == -1 )
			sleep(5);
		if( pid )
			exit(0);
		close(0); close(1); close(2);
		LogOpenSyslog(_DEF_MY_IDENT,LOG_MAIL);
	}
	else
	{
		LogOpenStderr();
	}
	SetLogLevel(LLevel);

	unlink(_DEF_MY_SOCKET);
	strlcpy(temp_socket, _DEF_DEF_PROTO, sizeof(temp_socket));
	strlcat(temp_socket, _DEF_MY_SOCKET, sizeof(temp_socket));
	if( smfi_setconn(temp_socket) == MI_FAILURE )
	{
		LogError(-1,"smfi_setconn(\'%s\') failed",_DEF_MY_SOCKET);
		return -1;
	}
	if( smfi_register(smfilter) == MI_FAILURE )
	{
		LogError(-1,"Cannot register filter");
		return -1;
	}

	LogDebug(8,"Starting main loop.");
	return smfi_main();
}

void parseCmdLine(int argc,char* argv[])
{
	char result;

	Daemonize=0;
	PrintPid=0;
	myRules=NULL;
	numRules=0;
	defaultRule=PASS_RULE;

	while( (result=getopt(argc,argv,"dpbF:v:")) != -1 )
	{
		if( result == 'd' )
			Daemonize=1;
		else if( result == 'p' )
			PrintPid=1;
		else if( result == 'v' )
			LLevel=atoi(optarg);
		else if( result == 'F' )
		{
			if( !(myRules=parse_rules(optarg,&numRules)) )
				exit(-1);
		}
		else if( result == 'b' )
			defaultRule=DROP_RULE;
	}

	return;
}

int ruleApplies(RULE_PTR rule,PRIV_DATA_PTR privData)
{
	int senderOK=0,rcptOK=0;
	NAMELIST_PTR ptr;

	if( rule->src_all )
		senderOK=1;
	else
		if( InHash(rule->src_hash,privData->from) != -1 )
			senderOK=1;

	if( rule->dest_all )
		rcptOK=1;
	else
	{
		ptr=(NAMELIST_PTR)(privData->root.next);
		while( ptr )
		{
			if( InHash(rule->dest_hash,ptr->name) != -1 )
			{
				rcptOK=1;
				break;
			}
			ptr=ptr->next;
		}
	}

	if( senderOK && rcptOK )
		return 1;
	else
		return 0;
}
