.\" $Id: aids_filter.man 2 2007-10-04 08:24:50Z rea $
.\" Copyright (C) 2001-2002 Eygene A. Ryabinkin, rea@rea.mbslab.kiae.ru.
.\"
.Dd March 30, 2002
.Dt aids_filter 8
.Os
.Sh NAME
.Nm aids_filter
.Nd checks incoming/outgoing mail for viruses.
.Sh SYNOPSIS
.Nm
.Op Fl F Ar config_file
.Op Fl v Ar verbosity_level
.Op Fl d
.Op Fl p
.Sh DESCRIPTION
The
.Nm
utility is a sendmail filter that checks incoming/outgoing mail
for viruses and takes appropriate actions. Currently it can be used with
DrWeb\(tm and Kaspersky's KAV\(tm antiviruses as a virus checking engines.
.Pp
Available command line options:
.Pp
.Bl -tag -width indent
.It Fl F Ar config_file
defines filter configuration file. Configuration file syntax is described in
section \fICONFIGURATION FILE\fP.
.It Fl v Ar verbosity_level
specifies a level of verbosity for debug messages. To enable developer debug
set it to 10.
.It Fl d
asks filter to daemonize, i.e. exit immediately, forking the child that will
process all further requests. All standart descriptors of filter will be closed
and all messages will be passed via syslog(3) using \fIMAIL\fP facility.
.It Fl p
will print process ID to standard output.
.El
.Sh CONFIGURATION FILE
.Nm
configuration file is a plain text file. Each line corresponds to a single
configuration parameter. Each line is preprocessed in the following way: all
spaces and tabs are converted to spaces, then multiple spaces are shrinked to
one space and starting and ending spaces are deleted from line.
All empty lines or lines starting with \fI#\fP symbol will be treated as
comments. Any other lines will be interpreted as configuration directives in
the form
.sp
parameter [value]
.sp
and all parameter names are case-insensitive but all values are case-sensitive.
Hereafter directives without values will be referred as \fIflags\fP.
.Pp
Configuration directives:
.Pp
.Bl -tag -width XXXXXXXXXXXX
.It \fBUseDrWeb\fP
flag that enables the use of DrWeb antivirus engine.
.It \fBUseKAV\fP
flag that enables the use of Kaspersky's KAV antivirus engine. Note, that
this and previous directives are mutually exclusive.
.It \fBHeuristic\fP
flag that turns on the heuristic analyser of antivirus engine.
.It \fBDaemonSocket\fP
fully qualified path to antivirus engine daemon socket. This socket must be
present at the startup of
.Nm
.It \fBSocket\fP
fully qualified path to
.Nm
socket that will be used by \fISendmail\fP to communicate with this filter.
.It \fBFileTemplate\fP
template for temporary files that will be created by mkstemp(3). The default
template is \fI/tmp/.virchkXXXXXX\fP.
.It \fBUserDB\fP
specifies the file with names of the users for which we will check
incoming/outgoing messages
for viruses. Each string of this file must be \fBfully qualified mail name\fP
of user, i.g. \fIjoe@email.org\fP, not just \fIjoe\fP.
If this directive isn't specified then all mail that goes through
us will be checked for viruses.
.It \fBWarnSender\fP
flag that allows filter to send a message to the sender of infected message.
Parameter \fBSenderMessage\fP also must be specified.
.It \fBSenderMessage\fP
fully qualified path to message that will be sent to sender of infected mail
if flag \fBWarnSender\fP was specified. See section \fIMESSAGE FORMAT\fP to
learn the format of this file.
.It \fBWarnRecipients\fP
flag that allows filter to send a message to the recipient(s) of infected
message. Parameter \fBRecipientMessage\fP also must be specified.
.It \fBRecipientMessage\fP
fully qualified path to message that will be sent to recipient(s) of infected
mail if flag \fBWarnRecipients\fP was specified. See section
\fIMESSAGE FORMAT\fP to learn the format of this file.
.It \fBWarnAdmin\fP
full mail address of user to whom will be sent a message if filter finds some
letter to be infected. Parameter \fBAdminMessage\fP also must be specified.
.It \fBAdminMessage\fP
fully qualified path to message that will be sent if filter finds some message
to be infected and if \fBWarnAdmin\fP was specified. See section
\fIMESSAGE FORMAT\fP to learn the format of this file.
.It \fBQuarantine\fP
path to directory, where infected messages will be stored. This directory
must be mode 0700.
.It \fBIncludeHeaders\fP
is specified, then the headers of infected message will be included.
.El
.Sh MESSAGE FORMAT
In this section the format of file with mail message will be explained. In
general, format of such file coincides with the format of ordinary mail
message, except for some special symbols substitution. If it make things
cleaner, I'll say that this message will be passed directly to \fBsendmail\fP,
invoked in it's normal (delivery, sendmail -bm) mode.
.Pp
Nevertheless, I'll explain general format of this file. Message consists of
\fIheaders\fP and the \fImessage body\fP; and this two sections are delimited
by empty line. Headers are lines in the form
.sp
        Header_name: value for this header
.sp
and there are some widely used header names:
.Bl -tag -width XXXXXXXXXX
.It \fBFrom\fP
specifies the sender of message.
.It \fBTo\fP
specifies the recipients of message.
.It \fBSubject\fP
specifies the subject of message.
.It \fBX-*\fP
any header name that starts with \fIX-\fP symbols is ignored by mail delivery
agents, but can be used by user mail agents and/or some special mail processing
programs.
.El
.Pp
As it had been said the end of headers block is denoted by single empty line and
after it follows the message itself. Message body has no special format, but
the following fact should be kept in mind: the line with single period in it
denotes the end of message, so if you want to make a line with a single period
on it just put two periods to this line.
.Pp
But before message will be passed by filter to \fBsendmail\fP some
substitutions in the spirit of printf(3) will be done. Here is the list of
substitutions:
.Bl -tag -width XXXXX
.It \fB%h\fP
the hostname of machine that runs
.Nm
\.
.It \fB%S\fP
the name of the sender of infected message.
.It \fB%R\fP
the name(s) of the recipient(s) of infected message.
.It \fB%r\fP
the reason of rejection of this message.
.El
.Sh FILTER INVOCATION AND CONFIGURATION OF SENDMAIL
.Nm
should be invoked before \fBsendmail\fP's invocation, but after antivirus
engine it will use. So, you must invoke your antivirus, then
.Nm
and then \fBsendmail\fP.
.Pp
But in order to pass the mail to this filter you must take some extra steps
to properly configure \fBsendmail\fP, namely you must rebuild it with
\fBMilter\fP support and edit configuration file to enable message passing to
\fBaids_filter\fP. Here is the instructions how it can be done.
.Pp
.po +0.2i
* Obtain sendmail version 8.12.1 or higher. Earlier versions are not
reccomended due to bug in \fBMilter\fP kernel.
.br
* Unpack sendmail to some directory and change to that directory.
Configure sendmail in the usual way (if don't know how, try to read files
INSTALL, README, sendmail/README, sendmail/SECURITY, devtools/README;
`...if else fails, try asking the human'), then add the following lines to file
devtools/Site/site.config.m4
.br
 APPENDDEF(`conf_sendmail_ENVDEF', `-D_FFR_MILTER=1')
.br
 APPENDDEF(`conf_sendmail_ENVDEF', `-DMILTER')
.br
 APPENDDEF(`conf_libmilter_ENVDEF', `-D_FFR_MILTER=1')
.br
* Build sendmail.
.br
* Choose an appropriate .mc file in cf/cf directory or write your own one.
Then add the following line to this file
.br
 INPUT_MAIL_FILTER(`StopAIDS', `S=local:/var/run/.aids_filter, F=R')
.br
For the explanation what it is read the file libmilter/README.
.br
* Install sendmail and it's configuration files, then set up spool directories
permissions if needed (see sendmail/SECURITY) and try to run sendmail with
antivirus and this filter.
.br
.po
Good luck!
.Sh EXAMPLES
An example of
.Nm
configuration file to use DrWeb as antivirus engine:
.br
.po +0.2i
UseDrWeb
.br
DaemonSocket /var/run/drwebd
.br
Socket /var/run/.aids_filter
.br
UserDB /usr/local/etc/aids_filter.users
.br
WarnAdmin root@email.org
.br
AdminMessage /usr/local/etc/aids_admin.message
.br
.po
.Pp
With such configuration filter will communicate with \fBdrwebd\fP through
/var/run/drwebd socket; will check mail only from/to users specified in
file /usr/local/etc/aids_filter.users; will send a letter to root@email.org
using message template from file /usr/local/etc/aids_admin.message in response
to any infected message.
.Pp
Example of message file:
.br
.po +0.2i
From: Antiviral.services
.br
Subject: Infected message from %S to %R.
.sp
.br
User %S tried to send an infected message to %R and it was rejected.
.br
The reason of rejection is `%r'.
.br
.po
.Sh BUGS
Not a bug, probably a feature: if filter will use \fBUserDB\fP file,
\fBWarnRecipients\fP will be enabled and there
will be infected message to multiple recipients, where among others will be
any user for whom we will check messages for viruses, all recipients will
be warned about virus and original infected message won't be delivered to
any of the users.
.br
Don't know any other bugs/features, please report if you'll find one.
.Sh AUTHOR
Eygene A. Ryabinkin, Russia, rea@rea.mbslab.kiae.ru.
