.\" $Id: log_filter.man 2 2007-10-04 08:24:50Z rea $
.\" Copyright (C) 2001-2002 Eygene A. Ryabinkin, rea@rea.mbslab.kiae.ru.
.\"
.Dd March 30, 2002
.Dt log_filter 8
.Os
.Sh NAME
.Nm log_filter
.Nd logs every mail message, has ability to set mail messages quotas for any
user and can mirror all mail to some directory.
.Sh SYNOPSIS
.Nm
.Op Fl F Ar log_file
.Op Fl L Ar log_string_format
.Op Fl Q Ar quotas_file
.Op Fl A Ar admin_mail
.Op Fl M Ar admin_message_template
.Op Fl R Ar reject_message_template
.Op Fl S Ar mirror_directory
.Op Fl H Ar CW_file
.Op Fl v Ar verbosity_level
.Op Fl m
.Op Fl d
.Op Fl p
.Sh DESCRIPTION
The
.Nm
utility is a sendmail filter that logs every incoming/outgoing message,
implements mail quotas mechanism and has the ability to put all messages to
some local directory to further analysis.
.Pp
Available command line options:
.Pp
.Bl -tag -width indent
.It Fl F Ar log_file
defines filter log file. Every incoming/outgoing message will be logged to
this file. Log file is string-oriented and single string format can be set
by command line switch \fB-L\fP.
.It Fl L Ar log_string_format
defines filter log file string format. Filter will perform the following
substitutions to log the actual message:
.Bl -tag -width XXXX
.It \fB%H\fP
the host name of machine this filter run on.
.It \fB%D\fP
date of message processing in standard ctime(3) form.
.br
Example: Thu Nov 24 18:22:48 1986.
.It \fB%F\fP
sender of the message.
.It \fB%R\fP
recipient(s) of the message.
.It \fB%S\fP
subject of the message.
.It \fB%L\fP
length of the message (headers and body) in bytes.
.It \fB%A\fP
\'A' if the message was accepted, 'R' if it was rejected.
.El
.It Fl Q Ar quotas_file
specifies the file with quotas for incoming mail messages. This file is
line-oriented
and each line consists of \fBfully qualified\fP mail user name
(joe@email.org, not just joe) any amount of spaces or tabs and single message
size limit in bytes. NOTE: if your machine is receiving mail for multiple
domains and you want to restrict the size of all incoming mail to some user
you must specify all domains in mail user names.
.It Fl A Ar admin_mail
specifies mail address of user to whom will be sent a mail if some message
will be rejected. \fB-M\fP must be also specified.
.It Fl M Ar admin_message_template
specifies message template that will be sent to user, specified by \fB-A\fP
option. Format of the message template file is described in section
\fIMESSAGE FORMAT\fP.
.It Fl R Ar reject_message_template
specifies reject message template and makes filter `noisy', i.e. if message
was rejected by some reason
there will be reply to sender of this message that warns him about message
rejection. Format of the message template file is described in section
\fIMESSAGE FORMAT\fP.
The default behaviour is to be `silent', because warning replies
can start `reply wars'.
.It Fl S Ar mirror_directory
specifies that incoming/outgoing messages will be saved to certain file in the
subdirectory of \fImirror_directory\fP. The latter file name will be
.br
    [In|Out]/user@host/DD.MM.YYYY_N
.br
where N is the serial number of message for given day. By default we will save
only mail from/for users at our host, whose name is given by hostname(1). If
you have more than one hostname and want to save messages from/for users at
that hosts flag \fB-H\fP will help you. NOTE: this directory's mode
must be 0700, see chmod(1).
.It Fl H Ar CW_file
specifies the name of the file with hostnames for which we will mirror mail
messages. This file is line-oriented, one line - one host. All characters
after '#' (and '#' itself) will be treated as comments. Filter behaviour is
simple: if sender/recipient domain coincides with name in \fICW_file\fP,
message will be mirrored.
.It Fl v Ar verbosity_level
specifies a level of verbosity for debug messages. To enable developer debug
set it to 10.
.It Fl m
will disable mmap(2)'ing of small messages (<8M). Though it's not recommended
because of perfomance decrease you can give up mmap(2)'ping.
.It Fl d
asks filter to daemonize, i.e. exit immediately, forking the child that will
process all further requests. All standart descriptors of filter will be closed
and all messages will be passed via syslog(3) using \fIMAIL\fP facility.
.It Fl p
will print process ID to standard output.
.El
.Sh MESSAGE FORMAT
In this section the format of file with mail message will be explained. In
general, format of such file coincides with the format of ordinary mail
message, except for some special symbols substitution. If it make things
cleaner, I'll say that this message will be passed directly to \fBsendmail\fP,
invoked in it's normal (delivery, sendmail -bm) mode.
.Pp
Nevertheless, I'll explain general format of this file. Message consists of
\fIheaders\fP and the \fImessage body\fP; and this two sections are delimited
by empty line. Headers are lines in the form
.sp
        Header_name: value for this header
.sp
and there are some widely used header names:
.Bl -tag -width XXXXXXXXXX
.It \fBFrom\fP
specifies the sender of message.
.It \fBTo\fP
specifies the recipients of message.
.It \fBSubject\fP
specifies the subject of message.
.It \fBX-*\fP
any header name that starts with \fIX-\fP symbols is ignored by mail delivery
agents, but can be used by user mail agents and/or some special mail processing
programs.
.El
.Pp
As it had been said the end of headers block is denoted by single empty line and
after it follows the message itself. Message body has no special format, but
the following fact should be kept in mind: the line with single period in it
denotes the end of message, so if you want to make a line with a single period
on it just put two periods to this line.
.Pp
But before message will be passed by filter to \fBsendmail\fP some
substitutions in the spirit of printf(3) will be done. For the list of
substitutions see section \fIDESCRIPTION\fP, option \fB-L\fP.
.Sh FILTER INVOCATION AND CONFIGURATION OF SENDMAIL
.Nm
should be invoked before \fBsendmail\fP's invocation.
So, you must invoke
.Nm
and, probably, some other filters and then \fBsendmail\fP.
.Pp
But in order to pass the mail to this filter you must take some extra steps
to properly configure \fBsendmail\fP, namely you must rebuild it with
\fBMilter\fP support and edit configuration file to enable message passing to
\fBlog_filter\fP. Here is the instructions how it can be done.
.Pp
.po +0.2i
* Obtain sendmail version 8.12.1 or higher. Earlier versions are not
reccomended due to bug in \fBMilter\fP kernel.
.br
* Unpack sendmail to some directory and change to that directory.
Configure sendmail in the usual way (if don't know how, try to read files
INSTALL, README, sendmail/README, sendmail/SECURITY, devtools/README;
`...if else fails, try asking the human'), then add the following lines to file
devtools/Site/site.config.m4
.br
 APPENDDEF(`conf_sendmail_ENVDEF', `-D_FFR_MILTER=1')
.br
 APPENDDEF(`conf_sendmail_ENVDEF', `-DMILTER')
.br
 APPENDDEF(`conf_libmilter_ENVDEF', `-D_FFR_MILTER=1')
.br
* Build sendmail.
.br
* Choose an appropriate .mc file in cf/cf directory or write your own one.
Then add the following line to this file
.br
 INPUT_MAIL_FILTER(`LogFilter', `S=local:/var/run/.log_filter, F=R')
.br
For the explanation what it is read the file libmilter/README.
.br
* Install sendmail and it's configuration files, then set up spool directories
permissions if needed (see sendmail/SECURITY) and try to run sendmail with
this filter.
.br
.po
Good luck!
.Sh EXAMPLES
Example of message file:
.br
.po +0.2i
From: Log.system
.br
Subject: Message from %F to %R was rejected.
.sp
.br
Mail from %F to %R with subject %S dated %D of size %L was rejected
.br
due to mail message quotas.
.br
.po
.Sh BUGS
Don't know any bugs/features, please report if you'll find one.
.Sh AUTHOR
Eygene A. Ryabinkin, Russia, rea@rea.mbslab.kiae.ru.
