/*
 * $Id: parse.c 13 2007-10-04 14:54:56Z rea $
 *
 * Copyright 2003-2007 Eygene A. Ryabinkin. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY EYGENE A. RYABINKIN "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EYGENE A. RYABINKIN OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#if !defined(lint)
static const char vcsid[] = "$Id: parse.c 13 2007-10-04 14:54:56Z rea $";
#endif

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include "stringops.h"
#include "parse.h"

#ifndef BUFLEN
#define BUFLEN 10240
#endif

int parse_config(FILE* fp,CONFIG_TOKENS_PTR tokens,long num)
/******************************************************************************
 * Function to process configuration file.                                    *
 ******************************************************************************/
{
	long i,n,size;
	char* ptr,line[BUFLEN];

	#ifdef DEBUG
	printf("parse_config(): starting with %ld tokens passed.\n",num);
	#endif

	fgets(line,BUFLEN,fp);
	while( !feof(fp) )
	{
		chomp(line);
		#ifdef DEBUG
		printf("Got line '%s'.\n",line);
		#endif
		i=0;
		while( isspace(line[i]) )
			i++;
		if( line[i] == '#' || line[i] == '\0' );
		/* Comment it was. */
		else
		{
			for(n=0;n<num;n++)
			{
				size=tokens[n].size;
				if (!strncasecmp(line+i, tokens[n].name, size)) {
					#ifdef DEBUG
					printf("Found token '%s' to be number %ld. Token type is %d.\n",
					tokens[n].name,n,tokens[n].type);
					#endif
					i+=size;
					if( tokens[n].type == CONFIG_TOKEN_FLAG )
					{
						*((int*)(tokens[n].addr))=1;
						#ifdef DEBUG
						printf("Flag set to 1.\n");
						#endif
					}
					else if( tokens[n].type == CONFIG_TOKEN_INT )
					{
						*((int*)(tokens[n].addr))=atoi(line+i);
						#ifdef DEBUG
						printf("Integer value set to %d.\n",*((int*)(tokens[n].addr)));
						#endif
					}
					else if( tokens[n].type == CONFIG_TOKEN_LONG )
					{
						*((long*)(tokens[n].addr))=atol(line+i);
						#ifdef DEBUG
						printf("Long integer value set to %ld.\n",
						*((long*)(tokens[n].addr)));
						#endif
					}
					else if( tokens[n].type == CONFIG_TOKEN_STRING )
					{
						while( isspace(line[i]) )
							i++;
						size = strlen(line+i);
						if( !(ptr=(char*)malloc((size+1)*sizeof(char))) )
						{
							fprintf(stderr,"%s,%d: unable to allocate memory.\n",__FILE__,__LINE__);
							return -1;
						}
						strlcpy(ptr, line+i, size + 1);
						*((char**)(tokens[n].addr))=ptr;
						#ifdef DEBUG
						printf("String value set to '%s'.\n",ptr);
						#endif
					}
					else;

					tokens[n].parsed=1;
				}
			}
		}
		fgets(line,BUFLEN,fp);
	}

	#ifdef DEBUG
	printf("parse_config(): done.\n");
	#endif

	return 0;
}
